/*-----------------------------------------------------------------------*
 Project: iWnn API Demo: Pseudo Dictionary Sample
 File:    ex_pseudo_ascii.h

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#ifndef _EX_PSEUDO_ASCII_H_
#define _EX_PSEUDO_ASCII_H_
#include <mw/iwnn/iwnnCTR.h>


#define IWNN_PSEUDOSET_ASCII_MAX    2 //Maximum count for the ASCII pseudo candidate type


#define NJ_TYPE_ASCII_HALF_TO_FULL        1                     //Pseudo-candidate type: ASCII to double-byte character
#define NJ_TYPE_ASCII_MAX_VALUE NJ_TYPE_ASCII_HALF_TO_FULL      //Pseudo candidate type maximum value
#define NJ_TYPE_ASCII_MIN_VALUE NJ_TYPE_ASCII_HALF_TO_FULL		//Pseudo candidate type minimum value

#define NJQG_EXP_HALF_TO_FULL_NUM    95 //Number of ASCII character code definitions



#define NJQG_HALF_LOWER_A    0x0061   //Lowercase ASCII "a"
#define NJQG_HALF_LOWER_Z    0x007A   //Lowercase ASCII "z"
#define NJQG_HALF_UPPER_A    0x0041   //Uppercase ASCII "A"
#define NJQG_HALF_UPPER_Z    0x005A   //Uppercase ASCII "Z"
#define NJQG_HALF_NUMERIC_0    0x0030 //ASCII numeric "0"
#define NJQG_HALF_NUMERIC_9    0x0039 //ASCII numeric "9"
#define NJQG_HALF_SYMBOL_SPACE  0x0020   //ASCII space (' ') 
#define NJQG_HALF_SYMBOL_SLASH  0x002F   //ASCII slash ('/')
#define NJQG_HALF_SYMBOL_COLON    0x003A //ASCII colon (':')
#define NJQG_HALF_SYMBOL_AT_MARK  0x0040 //ASCII at-mark ('@')
#define NJQG_HALF_SYMBOL_LEFT_SQUARE_BRACKET  0x005B //ASCII opening square bracket ('[')
#define NJQG_HALF_SYMBOL_GRAVE_ACCENT         0x0060 //ASCII grave accent ('`')
#define NJQG_HALF_SYMBOL_LEFT_CURLY_BRACKET   0x007B //ASCII opening curly bracket ('{')
#define NJQG_HALF_SYMBOL_OVERLINE             0x007E //ASCII tilde ('~')



//Structure for defining an ASCII pseudo candidate set
typedef struct {
    s16  count;                          //Number of registrations stored in type[]
    u8  type[IWNN_PSEUDOSET_ASCII_MAX];  //ASCII pseudo candidate type
} IWNN_PSEUDOSET_ASCII;

//Definition of entries in the reading character table
typedef struct {
    u16   src;       //Reading string
    u16   dst;       //Character after conversion
} IWNN_CONVERT_INFO;


//ASCII to double-byte code table definition
static const IWNN_CONVERT_INFO NjqgAsciiHalfToFullTable[NJQG_EXP_HALF_TO_FULL_NUM] = {
{0x20   /* */,       0x3000   /*　*/},
{0x21   /*!*/,       0xFF01   /*！*/},
{0x22   /*"*/,       0x201D   /*”*/},
{0x23   /*#*/,       0xFF03   /*＃*/},
{0x24   /*$*/,       0xFF04   /*＄*/},
{0x25   /*%*/,       0xFF05   /*％*/},
{0x26   /*&*/,       0xFF06   /*＆*/},
{0x27   /*'*/,       0x2019   /*’*/},
{0x28   /*(*/,       0xFF08   /*（*/},
{0x29   /*)*/,       0xFF09   /*）*/},
{0x2A   /***/,       0xFF0A   /*＊*/},
{0x2B   /*+*/,       0xFF0B   /*＋*/},
{0x2C   /*,*/,       0xFF0C   /*，*/},
{0x2D   /*-*/,       0xFF0D   /*－*/},
{0x2E   /*.*/,       0xFF0E   /*．*/},
{0x2F   /*slash*/,   0xFF0F   /*／*/},
{0x30   /*0*/,       0xFF10   /*０*/},
{0x31   /*1*/,       0xFF11   /*１*/},
{0x32   /*2*/,       0xFF12   /*２*/},
{0x33   /*3*/,       0xFF13   /*３*/},
{0x34   /*4*/,       0xFF14   /*４*/},
{0x35   /*5*/,       0xFF15   /*５*/},
{0x36   /*6*/,       0xFF16   /*６*/},
{0x37   /*7*/,       0xFF17   /*７*/},
{0x38   /*8*/,       0xFF18   /*８*/},
{0x39   /*9*/,       0xFF19   /*９*/},
{0x3A   /*:*/,       0xFF1A   /*：*/},
{0x3B   /*;*/,       0xFF1B   /*；*/},
{0x3C   /*<*/,       0xFF1C   /*＜*/},
{0x3D   /*=*/,       0xFF1D   /*＝*/},
{0x3E   /*>*/,       0xFF1E   /*＞*/},
{0x3F   /*?*/,       0xFF1F   /*？*/},
{0x40   /*@*/,       0xFF20   /*＠*/},
{0x41   /*A*/,       0xFF21   /*Ａ*/},
{0x42   /*B*/,       0xFF22   /*Ｂ*/},
{0x43   /*C*/,       0xFF23   /*Ｃ*/},
{0x44   /*D*/,       0xFF24   /*Ｄ*/},
{0x45   /*E*/,       0xFF25   /*Ｅ*/},
{0x46   /*F*/,       0xFF26   /*Ｆ*/},
{0x47   /*G*/,       0xFF27   /*Ｇ*/},
{0x48   /*H*/,       0xFF28   /*Ｈ*/},
{0x49   /*I*/,       0xFF29   /*Ｉ*/},
{0x4A   /*J*/,       0xFF2A   /*Ｊ*/},
{0x4B   /*K*/,       0xFF2B   /*Ｋ*/},
{0x4C   /*L*/,       0xFF2C   /*Ｌ*/},
{0x4D   /*M*/,       0xFF2D   /*Ｍ*/},
{0x4E   /*N*/,       0xFF2E   /*Ｎ*/},
{0x4F   /*O*/,       0xFF2F   /*Ｏ*/},
{0x50   /*P*/,       0xFF30   /*Ｐ*/},
{0x51   /*Q*/,       0xFF31   /*Ｑ*/},
{0x52   /*R*/,       0xFF32   /*Ｒ*/},
{0x53   /*S*/,       0xFF33   /*Ｓ*/},
{0x54   /*T*/,       0xFF34   /*Ｔ*/},
{0x55   /*U*/,       0xFF35   /*Ｕ*/},
{0x56   /*V*/,       0xFF36   /*Ｖ*/},
{0x57   /*W*/,       0xFF37   /*Ｗ*/},
{0x58   /*X*/,       0xFF38   /*Ｘ*/},
{0x59   /*Y*/,       0xFF39   /*Ｙ*/},
{0x5A   /*Z*/,       0xFF3A   /*Ｚ*/},
{0x5B   /*[*/,       0xFF3B   /*［*/},
{0x5C   /*\*/,       0xFFE5   /*￥*/},
{0x5D   /*]*/,       0xFF3D   /*］*/},
{0x5E   /*^*/,       0xFF3E   /*＾*/},
{0x5F   /*_*/,       0xFF3F   /*＿*/},
{0x60   /*`*/,       0x2018   /*‘*/},
{0x61   /*a*/,       0xFF41   /*ａ*/},
{0x62   /*b*/,       0xFF42   /*ｂ*/},
{0x63   /*c*/,       0xFF43   /*ｃ*/},
{0x64   /*d*/,       0xFF44   /*ｄ*/},
{0x65   /*e*/,       0xFF45   /*ｅ*/},
{0x66   /*f*/,       0xFF46   /*ｆ*/},
{0x67   /*g*/,       0xFF47   /*ｇ*/},
{0x68   /*h*/,       0xFF48   /*ｈ*/},
{0x69   /*i*/,       0xFF49   /*ｉ*/},
{0x6A   /*j*/,       0xFF4A   /*ｊ*/},
{0x6B   /*k*/,       0xFF4B   /*ｋ*/},
{0x6C   /*l*/,       0xFF4C   /*ｌ*/},
{0x6D   /*m*/,       0xFF4D   /*ｍ*/},
{0x6E   /*n*/,       0xFF4E   /*ｎ*/},
{0x6F   /*o*/,       0xFF4F   /*ｏ*/},
{0x70   /*p*/,       0xFF50   /*ｐ*/},
{0x71   /*q*/,       0xFF51   /*ｑ*/},
{0x72   /*r*/,       0xFF52   /*ｒ*/},
{0x73   /*s*/,       0xFF53   /*ｓ*/},
{0x74   /*t*/,       0xFF54   /*ｔ*/},
{0x75   /*u*/,       0xFF55   /*ｕ*/},
{0x76   /*v*/,       0xFF56   /*ｖ*/},
{0x77   /*w*/,       0xFF57   /*ｗ*/},
{0x78   /*x*/,       0xFF58   /*ｘ*/},
{0x79   /*y*/,       0xFF59   /*ｙ*/},
{0x7A   /*z*/,       0xFF5A   /*ｚ*/},
{0x7B   /*{*/,       0xFF5B   /*｛*/},
{0x7C   /*|*/,       0xFF5C   /*｜*/},
{0x7D   /*}*/,       0xFF5D   /*｝*/},
{0x7E   /*~*/,       0xFF5E   /*～*/},
};


//Dictionary interface extern declarations
NJ_EXTERN s16 NjexPseudoAsciiDic(IWNN_CLASS* iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE* message);


#endif /*_EX_PSEUDO_ASCII_H_*/
