/*-----------------------------------------------------------------------*
 Project: iWnn API Multi Language Demo: Main Function
 File:    main.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include <nn/applet.h>
#include <mw/iwnn/iwnnCTR.h>
#include "multidic_engine.h"
#include "multidic_com.h"
#include "fprintf.h"


#define DTOA_TAB   0x0900                     //TAB code
#define DTOA_LF    0x0a00                     //LF code
#define DTOA_CR    0x0d00                     //CR code

#define PROC_MODE_LEN 0x02                  //Language type string length
#define LANG_TYPE_LEN 0x02                  //Conversion mode string length

#define PROC_MODE_ANALYZE           0       //Predictive operation mode
#define PROC_MODE_CONNECT_ANALYZE   1       //Derived predictive operation mode
#define PROC_MODE_CONVERSION        2       //Conversion operation mode
#define DISP_ONE_LINE_CANDIDATE_NUM 10      //Number of candidates to display per line in the result file

s16 getLanguageStrWnn(wchar_t* languageStr, u32 languageType);    //Function for getting the language name
s16 getProcStr(wchar_t* procStr, u32 procType);                   //Function for getting the language name
    
nn::fnd::ExpHeap       s_DicHeap;               //Instance for dynamic memory allocation
nn::fs::FileOutputStream*   gout = NULL;        //Output file handle
MULTI_CLASS  languageEngineClass;                            //MULTI_CLASS process structure

wchar_t  candidate[NJ_MAX_CANDIDATE][NJ_MAX_LEN+NJ_TERM_LEN];  //Candidate region
wchar_t  reading[NJ_MAX_LEN+NJ_TERM_LEN];                      //Reading string
wchar_t  dispCandidate[NJ_MAX_LEN+NJ_TERM_LEN];                //Notation string
wchar_t  connectCandidate[NJ_MAX_LEN+NJ_TERM_LEN];             //Derived candidate string

wchar_t  procModeNum[PROC_MODE_LEN+1];               //Process mode setting options. 0: Prediction, 1: Derived prediction, 2: Conversion
wchar_t  languageTypeNum[LANG_TYPE_LEN+1];           //Language type setting option
wchar_t  resultBuff[NJ_MAX_LEN+NJ_TERM_LEN];         //Multiple phrase conversion output buffer
wchar_t  languageStr[NJ_MAX_LEN+NJ_TERM_LEN];        //Language name
wchar_t  procStr[NJ_MAX_LEN+NJ_TERM_LEN];            //Mode name  





/*
 
                  
                  
                  
                  
                  
                  
 

 
 */
extern "C" int nnMain()
{
    u32   languageType=0;      //Language type
    u32   preLanguageType;    //Previous language type
    u32   procMode=0;         //Processing mode     
    s16   segment;
    u8    testNum;
    wchar_t    *buf;
    wchar_t    *bufNext;
    u8    *filePtr;
    wchar_t    *ptr;
    s16   findIndex;
    s16   candidateCnt;
    s16   ret, i, j;
    u8     bom[2]={0xff,0xfe};

    //Enabling the applet library
    nn::applet::Enable();
    
    //fs initialization
    nn::fs::Initialize();

    //SD Card initialization
    nn::Result result = nn::fs::MountSdmc();
    if( result.IsFailure() ){
        return -1;
    }    

    internalMemset(&languageEngineClass, 0x00, sizeof(MULTI_CLASS));

    //Initialize the heap region
    s_DicHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    languageEngineClass.memHandle = (u8*)&s_DicHeap;        
    
    gout = new (nn::fs::FileOutputStream);
    if(!gout){
        nn::fs::Unmount("sdmc:");
        s_DicHeap.Finalize();
        return -1;
    }
    //Result output file (Result4.txt) initialization    
    gout->Initialize(FPRINTF_FILE_PATH,true);
    //Write BOM symbol
    gout->Write(bom,sizeof(bom),true);

    NN_LOG("  iWnn Demo: Demo start!\n");
    Fprintf(gout, L"Demo start!\n");      

    //************************
    //1. Loading input files
    //************************

    /****************Single record format of input files (UTF-16BE)**********************
    [LanguageTypeNo]<tab>[Mode]<tab>[InputReading]<tab>[ExpectedResult]<LF>
    
    [LanguageTypeNo]
            MULTI_LANG_TYPE_JP`MULTI_LANG_TYPE_CA_FULL   See: "multidic_engine.h"
    [Mode]
            0: Prediction, 1: Derived prediction, 2: Conversion
    [InputReading]
            Reading string
    [ExpectedResult]
            Expected result string
    (Ex.) Sample input file for specifying multiple languages
    26<tab>0<tab>app<tab>appellieren<LF>
    41<tab>0<tab>{p<tab>{p{<LF>
    14<tab>0<tab>re<tab><LF>
    *********************************************************************************/
    
    nn::fs::FileInputStream inFile(L"sdmc:/iWnn/input.txt");
    size_t fileSize = inFile.GetSize();    
    if(fileSize <= 0 ){
        Fprintf(gout, L"Invalid file size(%d)\n", fileSize);
        inFile.Finalize();
        gout->Finalize();
        nn::fs::Unmount("sdmc:");
        delete gout;
        s_DicHeap.Finalize();
        return -1;
    }
    filePtr = NULL;
    filePtr = (u8*)(s_DicHeap.Allocate(fileSize+1));
    if(filePtr == NULL ){
        Fprintf(gout, L"Failed to allocation \n");
        inFile.Finalize();
        gout->Finalize();
        nn::fs::Unmount("sdmc:");
        delete gout;
        s_DicHeap.Finalize();
        return -1;
    }
    internalMemset(filePtr, 0x00, sizeof(filePtr));    
    fileSize = inFile.Read(filePtr, fileSize);
    if(fileSize < 0 ){
        NN_LOG("  iWnn Demo: File Format Error!\n");
        Fprintf(gout,L"error:File Format Error\n");
        inFile.Finalize();
        s_DicHeap.Free(filePtr);
        gout->Finalize();
        nn::fs::Unmount("sdmc:");
        delete gout;
        s_DicHeap.Finalize();
        return -1;
    }
    inFile.Finalize();    

    buf = (wchar_t*)&filePtr[0];
    bufNext = buf;    
    preLanguageType = 0xff;
    testNum = 1;    
        
    while (*bufNext != 0x00){       
        buf = bufNext;               
        while (*bufNext != DTOA_LF){
            if (*bufNext == 0x00){
                NN_LOG("  iWnn Demo: File Format Error!\n");
                Fprintf(gout,L"error:File Format Error\n");
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }
            bufNext++;
        }
        *bufNext = 0x00;
        bufNext++;                         
        internalMemset(reading, 0x00, sizeof(reading));
        internalMemset(dispCandidate, 0x00, sizeof(dispCandidate));
        internalMemset(connectCandidate, 0x00, sizeof(connectCandidate));

        // Retrieves required information from input string 
        ptr = &buf[0];
        while (*ptr != 0x00) {
            // Load one line of data 
            j = 0;
            while (*ptr != DTOA_TAB) {
                // Read languageType 
                languageTypeNum[j] = *ptr;
                ptr++;
                j++;
            }
            languageTypeNum[j] = 0x00;            
            ptr++;            
            if (j > LANG_TYPE_LEN){
                NN_LOG("  iWnn Demo: File Format Error!\n");
                Fprintf(gout,L"error:File Format Error\n");
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;   
            }            
            
            j = 0;
            while (*ptr != DTOA_TAB) {
                // Read languageType 
                procModeNum[j] = *ptr;
                ptr++;
                j++;
            }
            procModeNum[j] = 0x00;
            ptr++;            
            if (j > PROC_MODE_LEN){
                NN_LOG("  iWnn Demo: File Format Error!\n");
                Fprintf(gout,L"error:File Format Error\n");
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;   
            }   

            languageType = internalAtoi(languageTypeNum);
            procMode = internalAtoi(procModeNum);
            if (getProcStr(procStr, procMode) < 0) {
                NN_LOG("  iWnn Demo: Proc Mode Error!\n");
                Fprintf(gout,L"error:Proc Mode Error\n");
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }

            j = 0;
            while (*ptr != DTOA_TAB) {
                // Get reading 
                reading[j] = *ptr;
                ptr++;
                j++;
            }
            reading[j] = 0x00;
            ptr++;

            j = 0;
            while ((*ptr != DTOA_TAB) && (*ptr != 0x00)) {
                // Get notation 
                dispCandidate[j] = *ptr;
                ptr++;
                j++;
            }
            dispCandidate[j] = 0x00;
            ptr++;
            
            if (procMode == PROC_MODE_CONNECT_ANALYZE) {
                // If Derived prediction is checked 
                j = 0;
                while ((*ptr != DTOA_TAB) && (*ptr != 0x00)) {
                    // Derived reading 
                    connectCandidate[j] = *ptr;
                    ptr++;
                    j++;
                }
                connectCandidate[j] = 0x00;
                ptr++;
                NN_LOG("  iWnn Demo: Proc Mode=PROC_MODE_CONNECT_ANALYZE \n");
           }else if(procMode==PROC_MODE_ANALYZE){
                NN_LOG("  iWnn Demo: Proc Mode=PROC_MODE_ANALYZE \n");

           }else if(procMode==PROC_MODE_CONVERSION){
                NN_LOG("  iWnn Demo: Proc Mode=PROC_MODE_CONVERSION \n");

           }
                       
            break;
        }
        //*********************************
        //    2. Setting languages and initializing iWnn   
        //*********************************
        if (preLanguageType != languageType) {           
            if (getLanguageStrWnn(languageStr, languageType) < 0) {
                NN_LOG("  iWnn Demo: Language Type Error!\n");
                Fprintf(gout,L"error:Language Type Error\n");
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }                       

            //Releasing the previous language
            if (preLanguageType != 0xff) {
                freeLanguage((u8*)&languageEngineClass, preLanguageType);
                 NN_LOG("  iWnn Demo: Setting Current Language to (langType=%d) \n", languageType);
            }
                    
            //MULTI_CLASS initialization
            if(initializeLanguage((u8*)&languageEngineClass) < 0 ){
                NN_LOG("  iWnn Demo: Initialize Language Error!\n");
                Fprintf(gout,L"error:Initialize Language Error\n");            
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }

            //Load language information
            if(loadLanguage((u8*)&languageEngineClass, languageType) < 0){
                NN_LOG("  iWnn Demo: Load Language Error!\n");
                Fprintf(gout,L"error:Load Language Error\n");            
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }

            //Language setting
            if(setLanguage((u8*)&languageEngineClass, languageType) < 0){
                NN_LOG("  iWnn Demo: Set Language Error!\n");
                Fprintf(gout,L"error:Set Language Error\n");            
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }
        }else{
            NN_LOG("  iWnn Demo: LanguageType(%d) is first language!\n",languageType);
        }
        
        
        
        NN_LOG("  iWnn Demo: (langType=%d procMode=%d) Process Start.\n", languageType, procMode); 
        Fprintf(gout, L"\n<====== demo %d ======>\n", testNum);
 
        Fprintf(gout, L"[Language]:");
        FprintUTF16LE(gout,languageStr);
        Fprintf(gout, L"\t\t[Mode]:");
        FprintUTF16LE(gout,procStr);
        Fprintf(gout, L"\t\n");        

        Fprintf(gout, L"[Input Reading]:");
        FprintUTF16BE(gout,reading);
        Fprintf(gout, L"\t[Expected Result]:");
        FprintUTF16BE(gout,dispCandidate);
        Fprintf(gout, L"\t");
        
        //*********************************
        //    3. Clearing the learning dictionary             
        //*********************************
        if(clearDic((u8*)&languageEngineClass, MULTI_DIC_TYPE_LEARN) < 0){
            NN_LOG("  iWnn Demo: Clear Learning Dic Error!\n");
            Fprintf(gout,L"error:Clear Learning Dic Error\n");            
            s_DicHeap.Free(filePtr);
            gout->Finalize();
            nn::fs::Unmount("sdmc:");
            delete gout;
            s_DicHeap.Finalize();
            return -1;
        }


        //*********************************
        //    4. Prediction/Connection prediction demo         
        //*********************************
        if(procMode != PROC_MODE_CONVERSION){ 
            // Get the prediction candidates 
            findIndex  = -1;
            candidateCnt = 0;
            internalMemset(candidate, 0x00, sizeof(candidate));
            ret = getConversion((u8*)&languageEngineClass, MULTI_MODE_PREDICTION, reading, 0, 0, NJ_MAX_LEN);
            if (ret < 0){
                Fprintf(gout,L"error:getConversion-%04x-\n", ret );
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }
            if (ret >= 0) {
                for (i = 0; i < NJ_MAX_CANDIDATE; i++) {
                    ret = getString((u8*)&languageEngineClass,candidate[i], 0, i,  sizeof(candidate[i]));
                    if (ret == 0) {
                        break;
                    } else if (ret < 0) {
                        Fprintf(gout,L"error:[%02d]getString-%04x-\n", i, ret);
                        s_DicHeap.Free(filePtr);
                        gout->Finalize();
                        nn::fs::Unmount("sdmc:");
                        delete gout;
                        s_DicHeap.Finalize();
                        return -1;
                    }
                    if (procMode == PROC_MODE_CONNECT_ANALYZE) {
                        // Check derived candidate
                        if (internalStrcmp(candidate[i], connectCandidate) == 0) {
                            ret = learnString((u8*)&languageEngineClass, 0, i);
                            if(ret < 0){
                                Fprintf(gout, L"error:learnString-%04x-\n", ret);
                                s_DicHeap.Free(filePtr);
                                gout->Finalize();
                                nn::fs::Unmount("sdmc:");
                                delete gout;
                                s_DicHeap.Finalize();
                                return -1;
                            }
                            break;
                        }
                    } else {
                        // Confirm prediction candidate order 
                        if (internalStrcmp(candidate[i], dispCandidate) == 0) {
                            findIndex = i+1;
                        }
                    }
                    candidateCnt++;
                }

                if (procMode == PROC_MODE_CONNECT_ANALYZE) {
                    findIndex  = 0;
                    candidateCnt = 0;
                    internalMemset(candidate, 0x00, sizeof(candidate));
                    ret = getConversion((u8*)&languageEngineClass, MULTI_MODE_PREDICTION, (wchar_t *)"", 0, 0, NJ_MAX_LEN);
                    if (ret < 0){
                        Fprintf(gout,L"error:getConversion-%04x-\n", ret );
                        s_DicHeap.Free(filePtr);
                        gout->Finalize();
                        nn::fs::Unmount("sdmc:");
                        delete gout;
                        s_DicHeap.Finalize();
                        return -1;
                    }
                    if (ret >= 0) {
                        for (i = 0; i < NJ_MAX_CANDIDATE; i++) {
                            ret = getString((u8*)&languageEngineClass, candidate[i],0, i,  sizeof(candidate[i]));
                            if (ret == 0) {
                                break;
                            } else if (ret < 0) {
                                Fprintf(gout,L"error:[%02d]getString-%04x-\n", i, ret);
                                s_DicHeap.Free(filePtr);
                                gout->Finalize();
                                nn::fs::Unmount("sdmc:");
                                delete gout;
                                s_DicHeap.Finalize();
                                return -1;
                            }

                            // Confirm prediction candidate order 
                            if (internalStrcmp(candidate[i], dispCandidate) == 0) {
                                findIndex = i+1;

                            }
                            candidateCnt++;
                        }
                    }
                    
                }
            }
            
            if (procMode == PROC_MODE_CONNECT_ANALYZE) {
                Fprintf(gout, L"[Confirmed Candidate]:");
                FprintUTF16BE(gout,connectCandidate);
            }
            Fprintf(gout, L"\n[Position]:%d\t\t[Number of Candidates]:%d", findIndex, candidateCnt);

            Fprintf(gout, L"\n[Candidates List]:");
            for (i = 0; i < candidateCnt; i++) {
                if ((i % DISP_ONE_LINE_CANDIDATE_NUM) == 0) {
                    Fprintf(gout, L"\n");
                }
                FprintUTF16BE(gout, candidate[i]);
                if ((candidateCnt-2) >= i) {
                    Fprintf(gout, L"\t");
                 }
            }

        //*********************************
        //     5. Multiple phrase conversion demo            
        //*********************************
        } else {
            candidateCnt = 0;
            // Perform multiple phrase conversion and get the phrase count 
            segment = getConversion((u8*)&languageEngineClass, MULTI_MODE_CONVERSION, reading, 0, 0, NJ_MAX_LEN);
            if (segment < 0){
                Fprintf(gout,L"error:getConversion-%04x-\n", segment );
                s_DicHeap.Free(filePtr);
                gout->Finalize();
                nn::fs::Unmount("sdmc:");
                delete gout;
                s_DicHeap.Finalize();
                return -1;
            }
            if (segment >= 0) {
                for (i = 0; i < NJ_MAX_CANDIDATE; i++) {
                    ret = getString((u8*)&languageEngineClass, candidate[i], 0, i, sizeof(candidate[i]));
                    if (ret == 0) {
                        break;
                    } else if (ret < 0) {
                        Fprintf(gout,L"error:[%02d]getString-%04x-\n", i, ret);
                        s_DicHeap.Free(filePtr);
                        gout->Finalize();
                        nn::fs::Unmount("sdmc:");
                        delete gout;
                        s_DicHeap.Finalize();
                        return -1;
                    }
                    candidateCnt++;
                }                
                Fprintf(gout, L"\n[Number of Segments]:%d\t", segment);

                // Get reading of first candidate 
                ret =  getString((u8*)&languageEngineClass,resultBuff, 0, 0,  sizeof(resultBuff));
                if (ret < 0) {
                    Fprintf(gout,L"error:[%02d]getString-%04x-\n", i, ret);
                    s_DicHeap.Free(filePtr);
                    gout->Finalize();
                    nn::fs::Unmount("sdmc:");
                    delete gout;
                    s_DicHeap.Finalize();
                    return -1;
                }
                Fprintf(gout, L"[Conversion Result]:");
                FprintUTF16BE(gout,resultBuff);
                // Combine remaining phrases 
                for (i = 1; i < segment; i++) {
                    ret = getString((u8*)&languageEngineClass,resultBuff, i, 0,  sizeof(resultBuff));
                    if (ret < 0) {
                        Fprintf(gout,L"error:[%02d]getString-%04x-\n", i, ret);
                        s_DicHeap.Free(filePtr);
                        gout->Finalize();
                        nn::fs::Unmount("sdmc:");
                        delete gout;
                        s_DicHeap.Finalize();
                        return -1;
                    }
                    Fprintf(gout, L"/");
                    FprintUTF16BE(gout,resultBuff);  
                }
                // Display first candidate list 
                Fprintf(gout, L"\n[Candidates List]:");
                for (i = 0; i < candidateCnt; i++) {
                    if ((i % DISP_ONE_LINE_CANDIDATE_NUM) == 0) {
                       Fprintf(gout, L"\n");
                    }
                    FprintUTF16BE(gout, candidate[i]);
                    if ((candidateCnt-2) >= i) {
                        Fprintf(gout, L"\t");
                    }
                }                                    
            }        
        }        
        Fprintf(gout, L"\n");
        internalMemset(buf, 0x00, sizeof(buf));
        preLanguageType = languageType; 
        testNum++;
        NN_LOG("  iWnn Demo: Demo finished.\n");     
    }    
    //**************************
    //6. Releasing language information and finalizing iWnn
    //**************************
    if(freeLanguage((u8*)&languageEngineClass, languageType) < 0 ){
        NN_LOG("  iWnn Demo: Free Language Error!\n");
        Fprintf(gout,L"error:Free Language Error\n");
        s_DicHeap.Free(filePtr);
        gout->Finalize();
        nn::fs::Unmount("sdmc:");
        delete gout;
        s_DicHeap.Finalize();
        return -1;

    }
    terminateLanguage((u8*)&languageEngineClass);    
    s_DicHeap.Free(filePtr);
    Fprintf(gout,L"demo end!\n");
    gout->Finalize();
    nn::fs::Unmount("sdmc:");
    delete gout;
    s_DicHeap.Finalize();
    NN_LOG("  iWnn Demo: Demo end.\n");
    while(1){}
}


/*
 

 
 

 
 */

s16 getLanguageStrWnn(wchar_t* languageStr, u32 languageType){    
    internalMemset(languageStr, 0x00, sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_LEN));
    if (languageType == MULTI_LANG_TYPE_JP_FULL) {
        internalStrcpy(languageStr, L"JA");
    } else if (languageType == MULTI_LANG_TYPE_EN_US_10S) {
        internalStrcpy(languageStr, L"EN_US_10S");
    } else if (languageType == MULTI_LANG_TYPE_EN_US_FULL) {
        internalStrcpy(languageStr, L"EN_US");
    } else if (languageType == MULTI_LANG_TYPE_EN_GB_10S) {
        internalStrcpy(languageStr, L"EN_GB_10S");
    } else if (languageType == MULTI_LANG_TYPE_EN_GB_FULL) {
        internalStrcpy(languageStr, L"EN_GB");
    } else if (languageType == MULTI_LANG_TYPE_CN_10S) {
        internalStrcpy(languageStr, L"CN_Pinyin_10S");
    } else if (languageType == MULTI_LANG_TYPE_CN_FULL) {
        internalStrcpy(languageStr, L"CN_Pinyin");
    } else if (languageType == MULTI_LANG_TYPE_TW_P10S) {
        internalStrcpy(languageStr, L"TW_Pinyin_10S");
    } else if (languageType == MULTI_LANG_TYPE_TW_PFULL) {
        internalStrcpy(languageStr, L"TW_Pinyin");
    } else if (languageType == MULTI_LANG_TYPE_TW_B10S) {
        internalStrcpy(languageStr, L"TW_Bopomofo_10S");
    } else if (languageType == MULTI_LANG_TYPE_TW_BFULL) {
        internalStrcpy(languageStr, L"TW_Bopomofo");
    } else if (languageType == MULTI_LANG_TYPE_KO_FULL) {
        internalStrcpy(languageStr, L"KO");
    } else if (languageType == MULTI_LANG_TYPE_FR_10S) {
        internalStrcpy(languageStr, L"FR_10S");
    } else if (languageType == MULTI_LANG_TYPE_FR_FULL) {
        internalStrcpy(languageStr, L"FR");
    } else if (languageType == MULTI_LANG_TYPE_DE_10S) {
        internalStrcpy(languageStr, L"DE_10S");
    } else if (languageType == MULTI_LANG_TYPE_DE_FULL) {
        internalStrcpy(languageStr, L"DE");
    } else if (languageType == MULTI_LANG_TYPE_ES_10S) {
        internalStrcpy(languageStr, L"ES_10S");
    } else if (languageType == MULTI_LANG_TYPE_ES_FULL) {
        internalStrcpy(languageStr, L"ES");
    } else if (languageType == MULTI_LANG_TYPE_PT_PT_10S) {
        internalStrcpy(languageStr, L"PT_PT_10S");
    } else if (languageType == MULTI_LANG_TYPE_PT_PT_FULL) {
        internalStrcpy(languageStr, L"PT_PT");
    } else if (languageType == MULTI_LANG_TYPE_PT_BR_10S) {
        internalStrcpy(languageStr, L"PT_BR_10S");
    } else if (languageType == MULTI_LANG_TYPE_PT_BR_FULL) {
        internalStrcpy(languageStr, L"PT_BR");
    } else if (languageType == MULTI_LANG_TYPE_IT_10S) {
        internalStrcpy(languageStr, L"IT_10S");
    } else if (languageType == MULTI_LANG_TYPE_IT_FULL) {
        internalStrcpy(languageStr, L"IT");
    } else if (languageType == MULTI_LANG_TYPE_RU_10S) {
        internalStrcpy(languageStr, L"RU_10S");
    } else if (languageType == MULTI_LANG_TYPE_RU_FULL) {
        internalStrcpy(languageStr, L"RU");
    } else if (languageType == MULTI_LANG_TYPE_NL_10S) {
        internalStrcpy(languageStr, L"NL_10S");
    } else if (languageType == MULTI_LANG_TYPE_NL_FULL) {
        internalStrcpy(languageStr, L"NL");
//2011/10/11 Commented out the Cantonese dictionary because it was deleted from the package.
//    } else if (languageType == MULTI_LANG_TYPE_HK_10S) {
//        internalStrcpy(languageStr, L"HK_10S");
//    } else if (languageType == MULTI_LANG_TYPE_HK_FULL) {
//        internalStrcpy(languageStr, L"HK");
    } else if (languageType == MULTI_LANG_TYPE_CA_10S) {
        internalStrcpy(languageStr, L"CA_10S");
    } else if (languageType == MULTI_LANG_TYPE_CA_FULL) {
        internalStrcpy(languageStr, L"CA");
    } else if (languageType == MULTI_LANG_TYPE_HK_CANGJIE) {//2011/12/05 Added support for the Cantonese dictionary (Cangjie) package
        internalStrcpy(languageStr, L"HK_CANGJIE");
    } else {
        return -1;
    }
    return 0;
}

/*
 

 
 

 
 */
s16 getProcStr(wchar_t* procStr, u32 procType){    
    internalMemset(procStr, 0x00, sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_LEN));
    switch (procType) {
        case PROC_MODE_ANALYZE:
            internalStrcpy(procStr, L"Prediction");
            break;
        case PROC_MODE_CONNECT_ANALYZE:
            internalStrcpy(procStr, L"Next Word Prediction");
            break; 
        case PROC_MODE_CONVERSION:
            internalStrcpy(procStr, L"Conversion");
            break;
        default:
            return -1; 
    }
    return 0;
}


