/*-----------------------------------------------------------------------*
 Project: iWnn API Multi Language Demo: Multi Language Framework
 File:    multidic_engine.h

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#ifndef _MULTI_ENGINE_H_
#define _MULTI_ENGINE_H_


#include <string.h>
#include <stdio.h>
#include <cstdlib>
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include <mw/iwnn/iwnnCTR.h>
#include "multidic_com.h"
#include "multidic_sys.h"


//* NULL string definition 
#define MULTI_CHAR_NUL      0x0000

// Language type definitions 
#define MULTI_LANG_TYPE_NONE        255       // None.                                   
#define MULTI_LANG_TYPE_JP_FULL       1       // Japanese (FullKey)
#define MULTI_LANG_TYPE_EN_US_10S     3       // English (10-key single)                    
#define MULTI_LANG_TYPE_EN_US_FULL    4       // English (FullKey)
#define MULTI_LANG_TYPE_EN_GB_10S     6       // English (10-key single)                          
#define MULTI_LANG_TYPE_EN_GB_FULL    7       // English (FullKey)
#define MULTI_LANG_TYPE_CN_10S        9       // Chinese (Simplified)(PinYin)(10-key single)         
#define MULTI_LANG_TYPE_CN_FULL       10      // Chinese (Simplified)(PinYin)(FullKey)     
#define MULTI_LANG_TYPE_TW_P10S       13      // Chinese (Traditional)(PinYin)(10-key single) */
#define MULTI_LANG_TYPE_TW_PFULL      14      // Chinese (Traditional)(PinYin)(FullKey)        
#define MULTI_LANG_TYPE_TW_B10S       16      // Chinese (Traditional)(Bopomofo)(PinYin)(10-key single) */
#define MULTI_LANG_TYPE_TW_BFULL      17      // Chinese (Traditional)(Bopomofo)(FullKey)      
#define MULTI_LANG_TYPE_KO_FULL       19      // Korean (FullKey)   
#define MULTI_LANG_TYPE_FR_10S        22      // French (10-key single)                     
#define MULTI_LANG_TYPE_FR_FULL       23      // French (FullKey)
#define MULTI_LANG_TYPE_DE_10S        25      // German (10-key single)                   
#define MULTI_LANG_TYPE_DE_FULL       26      // Dutch (FullKey)
#define MULTI_LANG_TYPE_ES_10S        28      // Spanish (10-key single)                  
#define MULTI_LANG_TYPE_ES_FULL       29      // Spanish (FullKey)
#define MULTI_LANG_TYPE_PT_PT_10S     31      // Portuguese - Portugal - (10-key single)                    
#define MULTI_LANG_TYPE_PT_PT_FULL    32      // Portuguese - Portugal - (FullKey)
#define MULTI_LANG_TYPE_PT_BR_10S     34      // Portuguese - Brazil - (10-key single)
#define MULTI_LANG_TYPE_PT_BR_FULL    35      // Portuguese - Brazil - (FullKey)
#define MULTI_LANG_TYPE_IT_10S        37      // Italian (10-key single)    
#define MULTI_LANG_TYPE_IT_FULL       38      // Italian (FullKey)
#define MULTI_LANG_TYPE_RU_10S        40      // Russian (10-key single)                   
#define MULTI_LANG_TYPE_RU_FULL       41      // Russian (FullKey)
#define MULTI_LANG_TYPE_NL_10S        43      // Dutch (10-key single)                     
#define MULTI_LANG_TYPE_NL_FULL       44      // Dutch (FullKey)
//2011/10/11 Commented out the Cantonese dictionary because it was deleted from the package.
//#define MULTI_LANG_TYPE_HK_10S        45      // Chinese (Kantonese)(10Key Single)
//#define MULTI_LANG_TYPE_HK_FULL       46      // Chinese (Kantonese)(PinYin)
#define MULTI_LANG_TYPE_CA_10S        47      // Quebecois (10-key single)    
#define MULTI_LANG_TYPE_CA_FULL       48      // Quebecois (FullKey)                   
#define MULTI_LANG_TYPE_HK_CANGJIE    49      // Chinese (Cantonese) (Cangjie)//2011/12/05 Support for the addition of the Cantonese (Cangjie) Dictionary to the package.



// Operation mode definitions 
#define MULTI_MODE_INIT          255   // Initialization mode       
#define MULTI_MODE_PREDICTION      0   // Prediction and conversion mode     
#define MULTI_MODE_CONVERSION      1   // Multiple phrase conversion mode   

// Dictionary type definitions 
#define MULTI_DIC_TYPE_NORMAL 0        // Normal dictionary           
#define MULTI_DIC_TYPE_USER   1        // User dictionary         
#define MULTI_DIC_TYPE_LEARN  2        // Learning dictionary           
#define MULTI_DIC_TYPE_MORPHO 2        // Morphological analysis learning dictionary 
#define MULTI_DIC_TYPE_SINGLE_KANJI 3  // Single kanji dictionary
#define MULTI_DIC_TYPE_FLASH  4        // FLASH dictionary          
#define MULTI_DIC_TYPE_LEARN_ADD  5    // Learning dictionary: Auxiliary information  

// Candidate type definitions 
#define MULTI_STR_TYPE_CANDIDATE   0   // Candidate list
#define MULTI_STR_TYPE_SEGMENT     1   // Phrase
#define MULTI_STR_TYPE_WORD   2        // Word information

// Search method definitions 
#define MULTI_CUR_OP_COMPLETE 0        // Complete match search
#define MULTI_CUR_OP_FORE     1        // Forward match search       

// Search candidate order definitions 
#define MULTI_CUR_MODE_FREQUENCY 0     // Frequency order
#define MULTI_CUR_MODE_READING   1     // Reading order

// Duplicate data display definition 
#define MULTI_DUPLICATE_OFF   0        // Do not display duplicate data 
#define MULTI_DUPLICATE_ON    1        // Display duplicate data   

// Fuzzy prediction definitions 
#define MULTI_FLEXIBLE_SEARCH_OFF   0  // Disable fuzzy searches       
#define MULTI_FLEXIBLE_SEARCH_ON    1  // Enable fuzzy search       
               
// Initialization function
s16 initializeLanguage(u8 *languageEngineClass);

// Finalization function
void terminateLanguage(u8 *languageEngineClass);

// Function for loading language settings
s16 loadLanguage(u8 *languageEngineClass, u8 language);

// Function for releasing language settings
s16 freeLanguage(u8 *languageEngineClass, u8 language);

// Function for specifying the language to use
s16 setLanguage(u8 *languageEngineClass, u8 language);

// Function for processing prediction and conversion and multiple phrase conversion
s16 getConversion(u8 *languageEngineClass, u8 mode, wchar_t *stroke, u8 devidePos, u8 strokeMin, u8 strokeMax);

// Function for getting candidate strings
s16 getString(u8 *languageEngineClass, wchar_t *buf,u16 segmentPos, u16 index,  u16 size);

// Function for clearing the learning dictionary, user dictionary, and morphological analysis learning dictionary
s16 clearDic(u8 *, u16);

// Function for learning
s16 learnString(u8 *languageEngineClass, u16 segmentPos, u16 index);

// Function for loading binary data
s32 internalReadBinary(u8 **,MULTI_CLASS *languageEngine, u8 *);

// Function for getting prediction candidates
s16 internalGetPrediction(MULTI_CLASS *languageEngine, u16 resultCnt);

// Function for getting homonyms (when getting all candidates)
s16 internalGetAllCandidate(MULTI_CLASS *languageEngine, u16 *,u16 );

// Function for initializing dictionary information
s16 internalCreateDic(MULTI_CLASS *languageEngine, u16);

// Function for checking dictionaries
s16 internalCheckDic(MULTI_CLASS *languageEngine, u16);

// Function for candidate learning
s16 internalSelect(MULTI_CLASS *languageEngine, s16, s16, u16);

// Function for loading dictionaries
s16 internalLoadDictionary (MULTI_CLASS *languageEngine, MULTI_LANG_INFO *languageInfo,
                                       u32 dicType,         u8 *dicName, u8 *exareaName,
                                       u8  *addInfo1,       u8 *addInfo2,
                                       u16 conversionBase,  u16 conversionHigh,
                                       u16 predictionBase,  u16 predictionHigh,
                                       u16 morphoBase,      u16 morphoHigh,
                                       u8  cache,           u8  limit,
                                       u8  dicCnt);    

s16     internalGetPredictionCN(MULTI_CLASS *languageEngine);
s16     internalGetPredictionCNFull(MULTI_CLASS *languageEngine);

// Function for converting from external code to internal code
s32 internalConvertInternal(MULTI_CLASS *languageEngine,  wchar_t *out,wchar_t *in);

// Function for converting from internal code to external code
s32 internalConvertExternal(MULTI_CLASS *languageEngine,  wchar_t *out, wchar_t *in);

// Function for fuzzy definitions
void    internalSetFlexibleSearchCharset(MULTI_CLASS *languageEngine);

// Function for filtering
s16     internalHashFilter(IWNN_CLASS* iwnn, IWNN_PHASE1_FILTER_MESSAGE* message);
s16     multidicSimplified10sFilter(IWNN_CLASS *iwnn, IWNN_PHASE1_FILTER_MESSAGE *message);
s16     multidicSimplifiedFullkeyFilter(IWNN_CLASS *iwnn, IWNN_PHASE1_FILTER_MESSAGE *message);
s16     internalHashFilterInit(MULTI_CLASS *languageEngine);
s16     internalHashFilterDestroy(MULTI_CLASS *languageEngine);
s16     internalAppendHash(MULTI_CLASS *languageEngine, wchar_t* string, s16 stringLen);
s32     internalComputeHash(wchar_t* string, s16 stringLen);
s32     internaToUpper(u8 caps, u8* pSrc, u8* pDst, s16 srcLen);
MULTI_STRING_ENTITY_T* internalFindHash(MULTI_CLASS *languageEngine, wchar_t* string, s16 stringLen);
MULTI_STRING_ENTITY_T* internalNextHash(MULTI_CLASS *languageEngine, s32 hash, s32 count);




#endif // _MULTI_ENGINE_H_ 
