/*-----------------------------------------------------------------------*
 Project: iWnn API Multi Language Demo: Same Candidate Filter/Dictionary Search Filter 
 File:    multidic_filter.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/

#include "multidic_engine.h"
#include "multidic_com.h"
#include "multidic_sys.h"
#include <mw/iwnn/iwnnCTR.h>
#include "multidic_table_cn.h"



/*
 

 

 
 
 */
#define MULTI_CHAR_IF_KEY_SIX(s)                       \
    ( (((u8)(((s) >> 8) & 0x00ff) == 0x36) &   \
      ((u8)(s) == 0x00)) ? 1 : 0 )

/*
 
 
 

 
 */
s16 internalHashFilterInit(MULTI_CLASS *languageEngine)
{
    internalMemset(languageEngine->hashEntity, 0, sizeof(languageEngine->hashEntity));
    internalMemset(languageEngine->hashString, 0, sizeof(languageEngine->hashString));
    return 0;
}

/*
 
 
 

 
 */
s16 internalHashFilterDestroy(MULTI_CLASS *languageEngine)
{
    return internalHashFilterInit(languageEngine);
}

/*
 
 
 
 

 
 
 */
s16 internalHashFilter(IWNN_CLASS* iwnn, IWNN_PHASE1_FILTER_MESSAGE* message)
{
    s16                ret     = 0;
    wchar_t                 string[NJ_MAX_RESULT_LEN + NJ_TERM_LEN];
    MULTI_CLASS*              languageEngine;
    MULTI_STRING_ENTITY_T*    pEntity = NULL;

    if ((iwnn == NULL) ||
        (message == NULL)) {
        return 1;
    }

    languageEngine = (MULTI_CLASS*)message->option;
    if(languageEngine->caps == MULTI_STRING_UPPER_NONE) {
        return 1;
    }
    if (languageEngine->candidateInfo.candidateCnt == 0) {
        /* If prediction candidates are not returned from the engine, set so that the filter does not operate
         
         */

        internaToUpper(languageEngine->caps, (u8*)message->string, (u8*)string, message->strLen);
        internalAppendHash(languageEngine, string, message->strLen);
        return 1;
    }
    
    // Compare from the last input string as target
    internaToUpper(languageEngine->caps, (u8*)message->string, (u8*)string, message->strLen);

    pEntity = internalFindHash(languageEngine, string, message->strLen);
    if(pEntity == NULL) {
        internalAppendHash(languageEngine, string, message->strLen);
        ret = 1;
    }
    return ret;
}

/*
 
 
 
 

 
 */
s32 internalComputeHash(wchar_t* string, s16 stringLen)
{
    s32   count;
    u32  total = 0;

    for (count = 0; count < stringLen; count++) {
        total += (u32)(string[count]);
    }
    return total % MULTI_STRING_HASH_MAX;
}

/*
 
 
 
 
 

 
 */
MULTI_STRING_ENTITY_T* internalFindHash(MULTI_CLASS *languageEngine, wchar_t* string, s16 stringLen)
{
    s32   hash    = internalComputeHash(string, stringLen);
    s32   count   = 0;
    MULTI_STRING_ENTITY_T*   subject = NULL;

    while( (subject = internalNextHash(languageEngine, hash, count)) != NULL) {
        if ((subject->stringLen == stringLen) &&
            ((nj_strncmp(subject->string, string, stringLen)) == 0)) {
            break;
        }
        count++;
    }
    return subject;
}

/*
 

 
 
 

 
 */
s16 internalAppendHash(MULTI_CLASS *languageEngine, wchar_t* string, s16 stringLen)
{
    MULTI_STRING_ENTITY_T entity;
    s32   count;
    s32   hash;
    s32   cursor;

    nj_strncpy(entity.string, string, stringLen);
    entity.stringLen = stringLen;

    for (count = 0; count < NJ_MAX_CANDIDATE; count++) {
        if((languageEngine->hashEntity[count].stringLen) == 0) {
            languageEngine->hashEntity[count] = entity;
            hash   = internalComputeHash(string, stringLen);
            cursor = languageEngine->hashString[hash].count;
            languageEngine->hashString[hash].entity[cursor] = &(languageEngine->hashEntity[count]);
            languageEngine->hashString[hash].count++;
            break;
        }
    }
    return 0;
}

/*
 

 
 
 

 
 */
MULTI_STRING_ENTITY_T* internalNextHash(MULTI_CLASS *languageEngine, s32 hash, s32 count)
{
    MULTI_STRING_ENTITY_T* pEntity = NULL;
    s32    cursor;

    if(languageEngine->hashString[hash].count > 0) {
        cursor = languageEngine->hashString[hash].count;

        if(count < cursor) {
            pEntity = languageEngine->hashString[hash].entity[count];
        }
    }
    return pEntity;
}

/*
 

 
                           
                           
                           
 
 
 

 
 */
s32 internaToUpper(u8 caps, u8* ptrSrc, u8* ptrDst, s16 srcLen)
{
    s32   i;
    s32   lop;
    u8    *out;
    u8    *src;
    s32   checkCnt = 18;
    u8   checkTbl[] = { 0x05, 0x07, 0x0D, 0x0F, 0x19, 
                             0x1B, 0x42, 0x44, 0x48, 0x53, 
                             0x59, 0x5B, 0x61, 0x65, 0x6F, 
                             0x7A, 0x7C, 0x7E };
    u8    checkFlag = 0;

    src = ptrSrc;
    out = ptrDst;
    
    for(i = 0; i < srcLen; i++) {
        if ((caps == MULTI_STRING_UPPER_TOP) &&
            (i >= 1)) {
            *out++ = *src++;
            *out++ = *src++;
            continue;
        }

        if (*src != 0x00) {
            if (*src == 0x01) {
                checkFlag = 0;
                *out++ = *src++;
                for (lop = 0; lop < checkCnt; lop++) {
                    if (*src == checkTbl[lop]) {
                        checkFlag = 1;
                        break;
                    }
                }
                if (checkFlag) {
                    *out   = *src - 0x01;
                    out++;
                    src++;
                    checkFlag = 0;
                }
            } else if (*src == 0x04) {
                *out++ = *src++;
                if ((*src >= 0x30) && (*src <= 0x4F)) {
                    *out = *src - 0x20;
                    out++;
                    src++;
                } else if (*src == 0x51) {
                    *out = *src - 0x50;
                    out++;
                    src++;
                } else {
                    *out++ = *src++;
                }
            } else {
                *out++ = *src++;
                *out++ = *src++;
            }
        } else {
            *out++ = *src++;
            if (((*src >= 0x61) && (*src <= 0x7A)) ||
                ((*src >= 0xE1) && (*src <= 0xFD))) {
                *out = *src - 0x20;
                out++;
                src++;
            } else if (*src == 0xFF) {
                *(out-1) = 0x01;
                *(out)   = 0x78;
                out++;
                src++;
            } else {
                *out++ = *src++;
            }
        }
    }
    *out++ = 0x00;
    *out++ = 0x00;
    return i;
}
/*
 
 
 
 

 
 

 */
s16 multidicSimplified10sFilter(IWNN_CLASS *iwnn, IWNN_PHASE1_FILTER_MESSAGE *message)
{
    wchar_t *pStr = NULL;
    u8 i, j, k;
    wchar_t tone4Char = 0;
    wchar_t num1Char = 0;
    wchar_t num5Char = 0;
    wchar_t rom1Char = 0;
    wchar_t rom4Char = 0;
    wchar_t fromChar = 0;
    wchar_t toChar = 0;
    wchar_t tmpChar = 0;
    u8 tone4Cnt = 0;
    u8 tone4Flg = 0;
    MULTI_CLASS* languageEngine;
    MULTI_SIMPLIFIED_FILTER_OPTION *filterOption = NULL;
    u8 flg = 0;
    wchar_t strNum1 = 0x0031;
    wchar_t strNum5 = 0x0035;
    wchar_t tmpStr = 0x0000;

    /* Check arguments */
    if ((iwnn == NULL) ||
        (message == NULL)) {
        return 1;
    }

    /* Get Option information */
    languageEngine = (MULTI_CLASS*)message->option;
    filterOption = (MULTI_SIMPLIFIED_FILTER_OPTION*)&(languageEngine->simplifiedOption);

    /* Compare the single kanji candidate length and additional information length */
    MULTI_CONV_TO_WCHAR(&num1Char, strNum1);
    MULTI_CONV_TO_WCHAR(&num5Char, strNum5);
    MULTI_CONV_TO_WCHAR(&rom1Char, MULTI_CHAR_ROM1);
    MULTI_CONV_TO_WCHAR(&rom4Char, MULTI_CHAR_ROM4);
    MULTI_CONV_TO_WCHAR(&tone4Char, languageEngine->tone4Char);

    /* When using tone toggle input */
    if ((tone4Char >= rom1Char) && (tone4Char <= rom4Char)) {

        /* Get candidate's additional information */
        pStr = message->additional;

        tmpStr = languageEngine->tone4Char - 0x212F;
        MULTI_CONV_TO_WCHAR(&tmpChar, tmpStr);

        /* Check up to the length of additional information */
        for (i = 0; i < message->additionalLen; i++) {
            /* Tone number determination */
            if ((*pStr >= num1Char) && (*pStr <= num5Char)) {
                tone4Cnt++;
                /* If it is the corresponding tone */
                if (*pStr == tmpChar) {
                    tone4Flg++;
                }

            }
            pStr++;
        }
        /* Because only single kanji are extracted,
           A multiple kanji compound is assumed if there is more than one tone */
        if ((tone4Cnt >= 2) || (message->strLen >= 2) ||
            (message->strokeLen != filterOption->strokeLen)) {
            /* Candidate not used */
            return 0;
        }

        /* If there is one matching tone */
        if (tone4Flg == 1) {
            /* Candidate is used */
            return 1;
        }

        /* Candidate not used */
        return 0;

    /* In all other cases */
    } else {
//      if (!filterOption->strokeFlg) {
        if ((!filterOption->strokeFlg) && (filterOption->separatorFlg == 0)) {
        /* This process is only performed when entered PinYin information exists */

            /* Get candidate's additional information */
            pStr = message->additional;

            /* Check up to the length of additional information */
            for (i = 0; i < message->additionalLen; i++) {
                if ((*pStr >= num1Char) && (*pStr <= num5Char)) {
                    tone4Cnt++;
                }
                pStr++;
            }
            /* A multiple kanji compound is assumed if there is more than one tone */
            if ((tone4Cnt >= 2) || (message->strLen >= 2)) {
                /* Candidate not used */
                return 0;
            }
//      } else if ((filterOption->strokeFlg) && (filterOption->separatorFlg > 0)) {
        } else if (filterOption->separatorFlg > 0) {
        /* If a delimiter character is input */
            /* Get candidate's additional information */
            pStr = message->additional;
            j = 0;

            /* If the length of the candidate string is the same as the number of delimiter characters */
            if (message->strLen <= filterOption->separatorFlg) {
                /* Candidate not used */
                return 0;
            }


            for (i = 0; i < filterOption->separatorFlg; i++) {
                /* If the length of additional information is equal to or greater than the input character string */
                if (message->additionalLen >= filterOption->separatorPos[i]) {
                    pStr = &message->additional[filterOption->separatorPos[i]];
                    /* Determine whether additional information includes tone information */
                    if ((*pStr >= num1Char) && (*pStr <= num5Char)) {
                        flg = 1;
                    } else {
                        /* The candidate is not used if it does not match the tone information */
                        flg = 0;
                        break;
                    }
                } else {
                /* If the length of additional information is less than the length of the input character string */
                    /* The candidate is not used because they do not match */
                    flg = 0;
                    break;
                }

                /* Check for the next delimiter character until the end of input */
                for (j = (u8)(filterOption->separatorPos[i] + 1); j < filterOption->separatorPos[i + 1]; j++) {
                    flg = 0;
                    /* Check number of fuzzy character sets (key assignments) */
                    for (k = 0; k < MULTI_FLEXBLE_CHARSET_CN_10S; k++) {
                        MULTI_CONV_TO_WCHAR(&fromChar, MULTI_CHARSET_TABLE_CN_10S[k].from[1]);
                        MULTI_CONV_TO_WCHAR(&toChar, MULTI_CHARSET_TABLE_CN_10S[k].to[1]);
                        /* Input strings and pre-conversion characters (10-key codes), reading characters and post-conversion characters (characters that can be entered by tapping keys) */
                        if ((filterOption->stroke[j] == fromChar) &&
                            (message->stroke[(j - (i + 1))] == toChar)) {
                            flg = 1;
                            break;
                        }
                    }
                    /* If comparison did not match the fuzzy character set */
                    if (!flg) {
                        /* Candidate not used */
                        return 0;
                    }
                }
            }

            /* If the check up to the number of delimiter characters has ended */
            if (flg) {
                /* Candidate is used */
                return 1;
            } else {
                /* Candidate not used */
                return 0;
            }
        }
    }

    /* Candidate is used */
    return 1;

}

/*
 
 
 
 

 
 
 */
s16 multidicSimplifiedFullkeyFilter(IWNN_CLASS *iwnn, IWNN_PHASE1_FILTER_MESSAGE *message) {

    wchar_t *pStr = NULL;
    u8 i;
    u8 cnt = 0;
    MULTI_CLASS* languageEngine;
    MULTI_SIMPLIFIED_FILTER_OPTION *filterOption = NULL;
    u8 flg = 0;
    u16 num1Char = 0;
    u16 num5Char = 0;
    wchar_t strNum1 = 0x0031;
    wchar_t strNum5 = 0x0035;

    /* Check arguments */
    if ((iwnn == NULL) ||
        (message == NULL)) {
        return 1;
    }

    /* Get option information and additional information */
    languageEngine = (MULTI_CLASS*)message->option;
    filterOption = (MULTI_SIMPLIFIED_FILTER_OPTION*)&(languageEngine->simplifiedOption);
    pStr = message->additional;

    /* Initialize the comparison string */
    MULTI_CONV_TO_WCHAR(&num1Char, strNum1);
    MULTI_CONV_TO_WCHAR(&num5Char, strNum5);

    /* If the input string differs from the candidate's notation string */
    if (filterOption->strokeLen != message->strLen) {
        /* 
         * If input in uppercase letters, to get a candidate with a length corresponding to the number of characters input, the candidate is not used if the input string differs from the candidate string notation.
         * 
         * 
         */
        return 0;
    }

    /* If a candidate does not include additional information */
    if (message->additionalLen == 0) {
        /* Candidate not used */
        return 0;
    }

    /* Checking the length of a candidate's additional information */
    for (i = 0; i < message->additionalLen; i++) {
        /* Tone or neutral tone information (character delimitation) is found */
        if ((*pStr >= num1Char) && (*pStr <= num5Char)) {
            /* If one character is input */
            if (filterOption->strokeLen == 1) {
                /* Set a flag */
                flg = 1;

            /* If two or more characters are input */
            } else {
                /* If the candidate reading string is longer than the location to be compared */
                /* 
                 * For example:
                 * Input characters: XX
                 * Candidate reading: xiexie
                 * Additional information: @_4@_5
                 * i: Additional information length, cnt: Number of tones or neutral tones.
                 * From the current additional information position, minus the count of tones or neutral tones, there should be the next input character, so check whether that character matches. 
                 * 
                 */
                if (message->strokeLen > (i - cnt)) {
                    /* If the character to be compared matches */
                    if ((message->stroke[i - cnt]) == filterOption->stroke[cnt + 1]) {
                        /* Set a flag */
                        flg = 1;
                    } else {
                        /* The flag is lowered and the candidate is not used. */
                        flg = 0;
                        break;
                    }
                }
            }
            /* Increment the count for the number of tones or neutral tone information */
            cnt++;
        }
        /* Advance through additional information */
        pStr++;
    }

    /* If the input characters do not match the number of tones or neutral tone information */
    if (cnt != filterOption->strokeLen) {
        /* 
         * Not used as a candidate because it differs from the string length to be displayed
         * 
         */
        return 0;
    }

    /* Additional information length check complete */
    if (flg) {
        /* Candidate is used */
        return 1;
    } else {
        /* Candidate not used */
        return 0;
    }
}
