/*-----------------------------------------------------------------------*
 Project: iWnn API Multi Language Demo: System API Wrapper
 File:    multidic_sys.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <cstdlib>
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include "multidic_sys.h"


#define CHAR_DIFF(a, b)                                              \
      ((s16)                                                         \
     ( (((u8*)(a))[0] == ((u8*)(b))[0])                     \
       ? (((u8*)(a))[1] - ((u8*)(b))[1])                    \
       : (((u8*)(a))[0] - ((u8*)(b))[0]) )                  \
     )
#define UTF16_NULL  0x0000
#define MAX_CONV_GRADE 5 //Maximum number of characters to be converted

/*
 

 

 
 */
u32  internalAtoi(const wchar_t *c)
{
    u32 len= internalStrlen((wchar_t*)c);
    u32 i;
    u32 ii;
    u32 ret=0;
    u32 grade;
    u16 tmp;
    u8 byteLow,byteHigh;
    if(len > MAX_CONV_GRADE){
        return 0;
    }
    for(i = 1; i <= len;i++){
        grade = 1;
        if(i < len){//Calculate from the 10s to the 10000s if other than the lowest digit
            for(ii = len ;ii > i;ii--){
                grade*=10;
            }
        }
        byteLow= (u8)((c[i-1] & (u16)0xff00) >> 8);
        byteHigh= (u8)(c[i-1] & (u16)0x00ff);
        tmp  = (u16)(byteHigh << 8) & (u16)0xff00;
        tmp |= (u16)byteLow;

        ret+=((tmp-0x0030)*grade);
    }
    return (u32)ret;
}

/*
 

 
 
 
 
 
 */
void  internalMemset(void *s, s32 c, u32 n)
{
    memset(s, c, n);
}

/*
 

 
 

 
 */
void* internalMallocation(u8 *memHandle, u32 n)
{
    void* ptr=NULL;
    if(!memHandle || n <= 0){
        return NULL;
    } 
    ptr = ((nn::fnd::ExpHeap *)memHandle)->Allocate(n);
    return ptr;
}


/*
 

 
 

 
 */
void  internalFree(u8 *memHandle, void *ptr)
{
    if(memHandle != NULL && ptr != NULL){
        ((nn::fnd::ExpHeap *)memHandle)->Free(ptr);
    }
}

/*
 
 
 
 
 
 
 
 
 */
MULTI_FILE*  internalFopen(MULTI_FILE* stream, s8 *filename, s8 *mode)
{
    if (strcmp((char*)mode, (char*)("rb")) == 0) 
    {
        ((nn::fs::FileStream *)stream)->Initialize((const char *)filename, nn::fs::OPEN_MODE_READ);
        return stream;   
    }
    else if (strcmp((char*)mode, (char*)("wb")) == 0){
        ((nn::fs::FileStream *)stream)->Initialize((const char *)filename, nn::fs::OPEN_MODE_WRITE);
        return stream;  
    }
    return NULL;
}



/*
 
 
 
 
 
 
 */
s32  internalFtell(MULTI_FILE *stream)
{
    if(!stream){
        return -1;
    }
    return ((nn::fs::FileStream *)stream)->GetSize();
}


/*
 
 
 
 
 
 
 
 
 
 */
s32 internalFread(void *ptr, s32 size, s32 n, MULTI_FILE *stream)
{
    if(!stream || !ptr || size <= 0 || n <= 0){
        return -1;
    }
    return (s32)((nn::fs::FileStream *)stream)->Read(ptr, (size*n)); 
}


/*
 
 
 
 
 
 */
void  internalFclose(MULTI_FILE *stream)
{
    if(stream != NULL){    
        ((nn::fs::FileStream *)stream)->Finalize();
    }
}


/*
 
 
 
 
 
 
 
 
 */
s32 internalStrcmp(wchar_t *s1, wchar_t *s2)
{
    while (*s1 == *s2) {
        if (*s1 == UTF16_NULL) {
            return (0);
        }
        s1++;
        s2++;
    }
    return CHAR_DIFF(s1, s2);

}
/*
 

 
 
 
 
 

 
 */
wchar_t *internalStrcpy(wchar_t *dst, wchar_t *src) {

    wchar_t *ret = dst;


    while (*src != UTF16_NULL) {
        *dst++ = *src++;
    }
    *dst = *src;
    return ret;
}


/*
 

 
 
 

 
 */
s32 internalStrlen(wchar_t *c) {

    s32 count = 0;

    while (*c++ != UTF16_NULL) {
        count++;
    }
    return count;
}
/*
 

 
 
 

 
 */
s32  internalSprintf (u8 *dicSett, const s8 *fmt, ...)
{
    va_list   args;
    s32 ret;
    va_start(args, fmt);
    ret = vsprintf((char*)dicSett, (const char*)fmt, args);
    va_end(args);
    return ret;
}
