/*-----------------------------------------------------------------------*
 Project: iWnn API Other APIs Demo: Main Function
 File:    main.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include <nn/applet.h>
#include <mw/iwnn/iwnnCTR.h>
#include "Fprintf.h"


#define MAX_DEMO_CANDIDATE 10                //Number of prediction candidates obtained
#define MAX_DEMO_STROKE_WORD_CANDIDATE 1     //Number of non-conversion candidates to obtain
#define MAX_MERGE_LIST 20      //Merge list count

#define MAX_LEARN_COUNT 1000  //Learning dictionary registration count
#define LEARN_DIC_SIZE (80 + 42*MAX_LEARN_COUNT) //Learning dictionary size

s32     readDic(u8 **dicData,char *dicName);
s32     loadReading(wchar_t *ptrReading,char *fileName, s32 stringSize);
void    releaseDic(void);
s16     demoAnalyzeOther( IWNN_CLASS *environment, wchar_t *reading, u16 select );
s16     demoState(IWNN_CLASS *environment, IWNN_STATE *state, wchar_t *reading);
s16     demoGetStrokeWord(IWNN_CLASS *environment, wchar_t *reading);
s16     demoChangeDicType(IWNN_CLASS *environment, IWNN_DIC_HANDLE handle, u8 direct);
s16     demoMergeWordList(IWNN_CLASS *environment, wchar_t *mergeReading1, wchar_t *mergeReading2);

u16 wcharStringLen(wchar_t *c);

wchar_t        analyzeReading[NJ_MAX_LEN+NJ_TERM_SIZE];    //Prediction candidate reading
wchar_t        stateReading[NJ_MAX_LEN+NJ_TERM_SIZE];      //State-adaptive prediction reading
wchar_t        strokeWordReading[NJ_MAX_LEN+NJ_TERM_SIZE]; //Non-conversion candidate reading
wchar_t        mergeReading1[NJ_MAX_LEN+NJ_TERM_SIZE];        //Reading 1 for candidate merge
wchar_t        mergeReading2[NJ_MAX_LEN+NJ_TERM_SIZE];        //Reading 2 for candidate merge
IWNN_CLASS      iwnnClass;                                 //iWnn class structure definition
IWNN_STATE      iwnnState;                                 //State setting
IWNN_OPTION      funcOption;                               //iWnn setting options
IWNN_DIC_SET      dicSet;                                  //Dictionary set structure
IWNN_MERGE_RESULT   resultList[MAX_MERGE_LIST];            //Merge candidate list
IWNN_FLASH_DIC_INFO    flashDicInfo[NJ_MAX_DIC];           //FLASH type dictionary information storage region
s16              dicCnt;                                   //Mounted dictionary count

nn::fnd::ExpHeap            dicHeap;       //Instance for dynamic memory allocation
nn::fs::FileOutputStream*   logFile=NULL;  //Log file output instance
nn::fs::FileStream*    inFileUbase1=NULL;  //Instance for working with Integrated Dictionary File 1.
nn::fs::FileStream*    inFileUbase2=NULL;  //Instance for working with Integrated Dictionary File 2.
u8* ptrUbase1=NULL;                          //Integrated Dictionary 1 handle pointer
u8* ptrUbase2=NULL;                          //Integrated Dictionary 2 handle pointer





/*
 
               
               
               
               
               
               
               
               
               

 

 
 */
extern "C" int nnMain()
{
    s32    fileSize;
    s16    ret = 0;
    u8     bom[2]={0xff,0xfe};


    //Enabling the applet library
    nn::applet::Enable();

    //fs initialization
    nn::fs::Initialize();

    //SD Card initialization
    nn::Result result = nn::fs::MountSdmc();
    if( result.IsFailure() ){
        return -1;
    }    

    //Result output file (Result5.txt) initialization
    logFile = new (nn::fs::FileOutputStream);
    if(!logFile){
        nn::fs::Unmount("sdmc:");
        return -1;
    }
    
    logFile->Initialize(FPRINTF_FILE_PATH,true);
     //Write BOM symbol
    logFile->Write(bom,sizeof(bom),true);
    Fprintf(logFile,L"demo start!\n");

    //Initialize the heap region
    dicHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    
    dicCnt = 0;
    memset(&iwnnClass, 0x00, sizeof(IWNN_CLASS));
    memset(&dicSet, 0x00, sizeof(IWNN_DIC_SET));
    memset(&flashDicInfo, 0x00, (NJ_MAX_DIC) * sizeof(IWNN_FLASH_DIC_INFO));    
    memset(analyzeReading, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(stateReading, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(strokeWordReading, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(mergeReading1, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(mergeReading2, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    
    //******************************
    // 1. Dictionary set structure settings       
    //******************************        
    NN_LOG("  iWnn Demo: Mounting dictionaries.\n");
    dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] = NULL;
    dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = NULL;
    dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = NULL;
    
    //Learning dictionary
    dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)dicHeap.Allocate(LEARN_DIC_SIZE);
        
    if (dicSet.dic[dicCnt].handle == NULL) {
        Fprintf(logFile, L"error:learn-dic mallocation\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    ret = mw::iwnn::IWNN::NjxCreateDic(dicSet.dic[dicCnt].handle, &iwnnClass, NJ_CREATE_DIC_TYPE_LEARN, LEARN_DIC_SIZE);
       
    if (ret < 0) {
        releaseDic();
        Fprintf(logFile, L"error:learn-dic create\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 1000;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 1000;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 1000;
    dicCnt++;

    //Integrated dictionary 1 (njubase1.a)
    inFileUbase1 = new(nn::fs::FileStream);
    inFileUbase1->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase1.a",nn::fs::OPEN_MODE_READ);
    
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase1);    
    
    if(fileSize <= 0){   
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }    
    
    ptrUbase1 = NULL;
    ptrUbase1  = (u8*)(dicHeap.Allocate(fileSize));
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase1, ptrUbase1, fileSize) < 0) {       
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
        
    } else {
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase1 after this will become unnecessary so, the file is closed here
    */
    inFileUbase1->Finalize();
    delete inFileUbase1;
    inFileUbase1=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 550;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 560;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;
    dicCnt++;
    
    //Integrated dictionary 2 (njubase2.a)
    inFileUbase2 = new(nn::fs::FileStream);
    inFileUbase2->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase2.a",nn::fs::OPEN_MODE_READ);
        
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase2);     
   
    if(fileSize <= 0){ 
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    ptrUbase2 = NULL;
    ptrUbase2 = (u8 *)dicHeap.Allocate(fileSize);
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase2, ptrUbase2, fileSize) < 0) {  
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;        
    } else {
        dicSet.dic[dicCnt].type   = NJ_DIC_HANDLE_TYPE_ON_FLASH;
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase2 after this will become unnecessary so, the file is closed here
    */
    inFileUbase2->Finalize();
    delete inFileUbase2;
    inFileUbase2=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 550;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 560;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;

    dicCnt++;

    //Auxiliary dictionary (njfzk.a)
    if( readDic( &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njfzk.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 500;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 500;
    dicCnt++;    

    //Single kanji dictionary (njtan.a)
    if( readDic( &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njtan.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;
    dicCnt++;

    //Extended no reading dictionary (njexreading.a)
    if( readDic( &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njexyomi.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 244;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 0;
    dicCnt++;

    // Implementation of pseudo-dictionary sample 1 
    dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
    dicSet.dic[dicCnt].extensionArea = NULL;
    dicSet.dic[dicCnt].handle = (u8*)(mw::iwnn::IWNN::NjexPseudoDic);
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_HIGH;
    dicSet.dic[dicCnt].searchCache = NULL;
    dicSet.dic[dicCnt].limit = 0;
    dicCnt++;

    // Implementation of pseudo-dictionary sample 2 
    dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
    dicSet.dic[dicCnt].extensionArea = NULL;
    dicSet.dic[dicCnt].handle = (u8*)(mw::iwnn::IWNN::NjexPredictionPseudoDic);
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_HIGH;
    dicSet.dic[dicCnt].searchCache = NULL;
    dicSet.dic[dicCnt].limit = 0;
    dicCnt++;

    //Rule dictionary (njcon.a)
    if( readDic( &dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION],"sdmc:/iWnn/dic/JA/njcon.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:rule dic open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
    dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
    dicSet.mode  =  NJ_CACHE_MODE_NONE;


    //******************************
    //2. Loading a reading string for demo
    //******************************
    NN_LOG("  iWnn Demo: Loading reading string for prediction.\n");
    if( (loadReading( analyzeReading,"sdmc:/iWnn/anlz.txt", sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze demo reading open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    NN_LOG("  iWnn Demo: Loading reading string for state prediction.\n");
    if( (loadReading( stateReading,"sdmc:/iWnn/state.txt", sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:state.txt open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    

    NN_LOG("  iWnn Demo: Loading reading string for getting stroke word list.\n");
    if( (loadReading( strokeWordReading,"sdmc:/iWnn/stroke_word.txt", sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:stroke_word.txt open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    NN_LOG("  iWnn Demo: Loading reading string for merge word list.\n");
    if( (loadReading( mergeReading1,"sdmc:/iWnn/merge1.txt", sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:merge1.txt open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    if( (loadReading( mergeReading2,"sdmc:/iWnn/merge2.txt", sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:merge2.txt open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    //******************************
    // 3. iWnn engine initialization       
    //******************************
    NN_LOG("  iWnn Demo: Initializing iWnn engine.\n");
    funcOption.autoConversionCnt = NJ_OPTION_AUTO_CONVERSION_CNT;
    mw::iwnn::IWNN::NjxInit(&iwnnClass, &funcOption);   
    iwnnClass.dicSet = dicSet;


    //*******************************
    // 4. State-adaptive predictive text demo 
    //*******************************
    NN_LOG("  iWnn Demo: State prediction demo.\n");
    ret = demoState( &iwnnClass, &iwnnState, stateReading);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    //******************************
    // 5. Demo of standard predictive conversion and other functions   
    //******************************
    NN_LOG("  iWnn Demo: Prediction and other APIs demo.\n");
    ret = demoAnalyzeOther( &iwnnClass, analyzeReading, 3);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    //******************************
    // 6. Demo of getting non-conversion candidates
    //******************************
    NN_LOG("  iWnn Demo: Getting stroke word list demo.\n");
    ret = demoGetStrokeWord(&iwnnClass, strokeWordReading);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:stroke word demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    //******************************
    // 7. Merge candidate list demo             
    //******************************
    NN_LOG("  iWnn Demo: Merge word list demo.\n");
    ret = demoMergeWordList(&iwnnClass, mergeReading1, mergeReading2);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:Merge list demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    //******************************
    // 8. Change dictionary type demo          
    //******************************
    NN_LOG("  iWnn Demo: Change dictionary type demo.\n");
    ret = demoChangeDicType(&iwnnClass, iwnnClass.dicSet.dic[0].handle, 0);
    if( ret != 0 ){
        releaseDic();
        Fprintf(logFile, L"error:change dic type demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    //******************************
    // 9. Dictionary unmount           
    //******************************
    NN_LOG("  iWnn Demo: Unmounting dictionaries.\n");
    releaseDic();


    Fprintf(logFile,L"demo end!\n");
    logFile->Finalize();    
    nn::fs::Unmount("sdmc:");
    delete logFile;
    dicHeap.Finalize();
    NN_LOG("  iWnn Demo: Demo end.\n");
    while(1){}
}


/*
 
               
               
               
               

 
 

 
 */
s32 readDic(u8 **dicData,char *dicName )
{         
    size_t  fileSize;

    nn::fs::FileInputStream inFile(dicName);
    fileSize = inFile.GetSize();
 
    if(fileSize <= 0 ){
        inFile.Finalize();
        Fprintf(logFile, L" Invalid file size(%d)\n", fileSize);
        return -1;
    }
    
    *dicData = (IWNN_DIC_HANDLE)dicHeap.Allocate(fileSize);
    if(*dicData == NULL ){
        inFile.Finalize();
        Fprintf(logFile, L" Failed to allocation \n");
        return -1;
    }

    fileSize = (s32)inFile.Read(*dicData, fileSize);    
    inFile.Finalize();
    return(fileSize);
}

/*
 
               
               
               

 
 
 

 
 */
s32 loadReading( wchar_t *ptrReading,char *fileName,s32 stringSize){

    // Check arguments
    if ( ptrReading == NULL ){
        Fprintf(logFile, L" parameter error(ptrReading = NULL)\n");
        return -1;
    }

    nn::fs::FileInputStream inFile(fileName);
    size_t fileSize = inFile.GetSize();
    if (fileSize >= stringSize) {
        Fprintf(logFile,L" Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }
        
    if (fileSize <= 0) {
        Fprintf(logFile, L"Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }

    fileSize = inFile.Read(ptrReading, fileSize);
    inFile.Finalize();
    return(fileSize);
}

/*
 
               
               
               

 

 
 */
void releaseDic(void){
    u16 i;

    if(ptrUbase1 != NULL){
        dicHeap.Free(ptrUbase1);        
        dicSet.dic[1].handle=NULL;
        ptrUbase1 = NULL;
    }
    if(ptrUbase2 != NULL){
        dicHeap.Free(ptrUbase2);        
        dicSet.dic[2].handle=NULL;
        ptrUbase2 = NULL;
    }

    for(i = 0; i < dicCnt;i++){
        if(dicSet.dic[i].handle != NULL &&
           dicSet.dic[i].type== NJ_DIC_HANDLE_TYPE_NRM){
            dicHeap.Free(dicSet.dic[i].handle);
        }
    }
    if( dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] != NULL){
        dicHeap.Free(dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]);
    }
    if(inFileUbase1!=NULL){
        inFileUbase1->Finalize();
        delete inFileUbase1;
    }    

    if(inFileUbase2!=NULL){
        inFileUbase2->Finalize();
        delete inFileUbase2;
    }
}

/*
 
               
               
               
               
               

 
 
 

 
 */
s16 demoState(IWNN_CLASS *environment, IWNN_STATE *state, wchar_t *reading)
{
    IWNN_RESULT    result[MAX_DEMO_CANDIDATE];
    wchar_t        readingUtf16[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t        buf[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];
    s16    ret;
    wchar_t     *ptrReading;
    u16    i;
    IWNN_ANALYZE_OPTION    analyzeOption;    

    Fprintf(logFile,L"-------------------------------------------------------\n");
    Fprintf(logFile,L"State-Adapted-Prediction Demo -Start-\n");

    if ((environment == NULL)||(state == NULL) || (reading == NULL)) {
        return -1;
    }

    if(wcharStringLen(reading) > NJ_MAX_LEN){
        return -1;
    }
    
    // Get default setting value
    ret=mw::iwnn::IWNN::NjxGetState(environment, state);
    if(ret < 0 ){
        Fprintf(logFile,L"error:NjxGetState-%04x-\n", ret);
        return -1;
    }

    // Set state to morning
    state->system[NJ_CATEGORY_TIME_MORNING] += 200;
    if (state->system[NJ_CATEGORY_TIME_MORNING] > 1000) {
        state->system[NJ_CATEGORY_TIME_MORNING] = 1000;
    }

    // Update state-adapted setting value
    ret=mw::iwnn::IWNN::NjxSetState(environment, state);
    if(ret < 0){
        Fprintf(logFile,L"error:NjxSetState-%04x-\n", ret);
        return -1;
    }
    memset(state, 0x00, sizeof(IWNN_STATE));
    
    // Get new setting value
    ret=mw::iwnn::IWNN::NjxGetState(environment, state);
    if(ret < 0){
        Fprintf(logFile,L"error:NjxGetState-%04x-\n", ret);
        return -1;
    }
    
    Fprintf(logFile,L"State-Adapted-Prediction Setting Values:\n");
    for (i = 0; i < NJ_MAX_STATE; i++){
        Fprintf(logFile, L"[%d]:%d ", i, state->system[i]);
    }
    Fprintf(logFile,L"\n");

    //Prediction candidate option settings
    analyzeOption.mode = NJ_NO_MULTI_CONVERSION|NJ_NO_SINGLE_CONVERSION;
    analyzeOption.forecastLearnLimit = 20;
    analyzeOption.forecastLimit = 100;
    analyzeOption.charMin = 0;
    analyzeOption.charMax = 40;

    memset( readingUtf16, 0x00, sizeof(readingUtf16) );

    
    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,reading);
    Fprintf(logFile,L"\n");

    // Get predictive conversion candidates
    Fprintf(logFile,L" Prediction List\n");
    memcpy(&readingUtf16[0], reading, sizeof(readingUtf16));
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {
        memset(&result[i], 0x00, sizeof(IWNN_RESULT));
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyzeOption);
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));

            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;    
        } else {
            Fprintf(logFile,L"error:[%02d]nj_analyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");
    Fprintf(logFile,L"State-Adapted-Prediction Demo -End-\n");
    Fprintf(logFile,L"-------------------------------------------------------\n"); 
    return i;
}

/*
 
               
               
               
               
               
               
               

 
 
 

 
 */
s16 demoAnalyzeOther( IWNN_CLASS *environment, wchar_t *reading, u16 select)
{
    IWNN_RESULT    result[MAX_DEMO_CANDIDATE];
    wchar_t        readingUtf16[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t        readingUtf16_2[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t        buf[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];
    u8    stemType[(NJ_MAX_LEN+NJ_TERM_SIZE)*2*2];
    u8    ancillaryType[(NJ_MAX_LEN+NJ_TERM_SIZE)*2*2];
    s16    ret;
    wchar_t     *ptrReading;
    u16    i;
    IWNN_DIC_HANDLE   dicHandle;

    IWNN_ANALYZE_OPTION    analyzeOption;    

    if ((environment == NULL)||(reading == NULL)) {
        return -1;
    }
    if(wcharStringLen(reading) > NJ_MAX_LEN){
        return -1;
    }

    if ((select >= NJ_MAX_CANDIDATE) || (select >= MAX_DEMO_CANDIDATE)){
        return -1;
    }

    //Prediction candidate option settings
    analyzeOption.mode = NJ_NO_MULTI_CONVERSION|NJ_RELATION_ON|NJ_NO_READING_ON;
    analyzeOption.forecastLearnLimit = 20;
    analyzeOption.forecastLimit = 100;
    analyzeOption.charMin = 0;
    analyzeOption.charMax = 40;

    memset( readingUtf16, 0x00, sizeof(readingUtf16) );
    memset( readingUtf16_2, 0x00, sizeof(readingUtf16_2) );

    Fprintf(logFile,L"-------------------------------------------------------\n");
    Fprintf(logFile,L"Undo Demo -Start-\n");

    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,reading);
    Fprintf(logFile,L"\n");

    // Get predictive conversion candidates
    Fprintf(logFile,L" Prediction List\n");
    memcpy(&readingUtf16[0], reading, sizeof(readingUtf16));
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {

        memset(&result[i], 0x00, sizeof(IWNN_RESULT));
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyzeOption);       
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;    
        } else {
            Fprintf(logFile,L"error:[%02d]nj_analyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");


    // Get dictionary handle
    dicHandle = mw::iwnn::IWNN::NjxGetDicHandle(environment, &result[0]);
    if (dicHandle == 0) {
        Fprintf(logFile,L"error:[%02d]NjxGetDicHandle-%04x-\n", i, dicHandle);
        return -1;
    } else {
        Fprintf(logFile,L"NjxGetDicHandle Demo -OK-\n");
    }

    //Merge learning dictionaries
    ret = mw::iwnn::IWNN::NjxManageLearnDic(environment, NJ_MLD_OPERATION_COMMIT);
    if (ret < 0) {
        Fprintf(logFile,L"error:NjxManageLearnDic-%04x-\n", ret);
        return -1;
    } else {
        Fprintf(logFile,L"NjxManageLearnDic Demo -OK-\n");
    }

    
    Fprintf(logFile,L" Select [%2d]\n",select);
    ret = mw::iwnn::IWNN::NjxSelect(environment, &result[select], &result[select], 0);
    if (ret < 0) {
        Fprintf(logFile,L"error:NjxSelect-%04x-\n",  ret);
        return ret;
    }




    //Get derived prediction candidate
    Fprintf(logFile,L" Next Word Prediction List\n");
    readingUtf16_2[0] = '\0';
    ptrReading = &readingUtf16_2[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyzeOption);
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }

            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;
        } else {
            Fprintf(logFile,L"error:[%02d]nj_analyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");

    //Get character type
    ret = mw::iwnn::IWNN::NjxGetCharType(stemType, ancillaryType, environment, &result[0]);
    if (ret < 0) {
        Fprintf(logFile,L"error:[%02d]NjxGetCharType-%04x-\n", i, dicHandle);
        return -1;
    } else {
        Fprintf(logFile,L"NjxGetCharType Demo -OK-\n");
    }

    //Undo learning
    ret=mw::iwnn::IWNN::NjxUndo(environment, 1);
    if(ret < 0){
        Fprintf(logFile,L"error:NjxUndo-%04x-\n", ret);
        return -1;
    }

    // Prediction 
    Fprintf(logFile,L" Prediction List\n");
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyzeOption);
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;
        } else {
            Fprintf(logFile,L"error:[%02d]nj_analyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");
    Fprintf(logFile,L"Undo Demo -End-\n\n");
    Fprintf(logFile,L"-------------------------------------------------------\n");
    return i;
}

/*
 
               
               

 
 

 
 */
s16 demoGetStrokeWord(IWNN_CLASS *environment, wchar_t *reading)
{
    IWNN_RESULT    result[MAX_DEMO_CANDIDATE];
    wchar_t        readingUtf16[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t        buf[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];
    s16    ret;
    wchar_t     *ptrReading;
    u16    i;

    if ((environment == NULL)||(reading == NULL)) {
        return -1;
    }
    if(wcharStringLen(reading) > NJ_MAX_LEN){
        return -1;
    }
    Fprintf(logFile,L"-------------------------------------------------------\n");
    Fprintf(logFile,L"Get Stroke Word Demo -Start-\n");
    

    memset( readingUtf16, 0x00, sizeof(readingUtf16) );

    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,reading);
    Fprintf(logFile,L"\n");
    
    // Get number of non-conversion candidates
    Fprintf(logFile,L" Prediction List\n");
    memcpy(&readingUtf16[0], reading, sizeof(readingUtf16));
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_STROKE_WORD_CANDIDATE; i++) {
        memset(&result[i], 0x00, sizeof(IWNN_RESULT));
        ret = mw::iwnn::IWNN::NjxGetStrokeWord(environment, &result[i], ptrReading);
        if (ret > 0) {
            // Get non-converted candidate string
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));

            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;    
        } else {
            Fprintf(logFile,L"error:[%02d]nj_get_stroke_word-%04x-\n", i, ret);
            return ret;
        }
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");
    Fprintf(logFile,L"Get Stroke Word Demo -End-\n");
    Fprintf(logFile,L"-------------------------------------------------------\n"); 
    return i;

}
/*
 
               
               
               
               

 
 
 

 
 */
s16 demoMergeWordList(IWNN_CLASS *environment, wchar_t *mergeReading1, wchar_t *mergeReading2){
    IWNN_RESULT    result1[MAX_DEMO_CANDIDATE];
    IWNN_RESULT    result2[MAX_DEMO_CANDIDATE];
    wchar_t        readingUtf16[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t        readingUtf16_2[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t        buf[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];
    s16    ret;
    wchar_t     *ptrReading;
    u16    i;

    IWNN_ANALYZE_OPTION    analyzeOption;    

    if ((environment == NULL)||(mergeReading1 == NULL)||(mergeReading2 == NULL)) {
        return -1;
    }

    if(wcharStringLen(mergeReading1) > NJ_MAX_LEN){
        return -1;
    }
    if(wcharStringLen(mergeReading2) > NJ_MAX_LEN){
        return -1;
    }

    //Prediction candidate option settings
    analyzeOption.mode = NJ_NO_MULTI_CONVERSION|NJ_RELATION_ON|NJ_NO_READING_ON;
    analyzeOption.forecastLearnLimit = 20;
    analyzeOption.forecastLimit = 100;
    analyzeOption.charMin = 0;
    analyzeOption.charMax = 40;

    memset( readingUtf16, 0x00, sizeof(readingUtf16) );
    memset( readingUtf16_2, 0x00, sizeof(readingUtf16_2) );

    Fprintf(logFile,L"-------------------------------------------------------\n");
    Fprintf(logFile,L"Get Merge List Demo -Start-\n");

    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,mergeReading1);
    Fprintf(logFile,L"\n");

    // Get predictive candidate list 1
    Fprintf(logFile,L" Prediction List\n");
    memcpy(&readingUtf16[0], mergeReading1, sizeof(readingUtf16));
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {

        memset(&result1[i], 0x00, sizeof(IWNN_RESULT));
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result1[i], NULL, ptrReading, &analyzeOption);       
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result1[i], sizeof(buf));
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;    
        } else {
            Fprintf(logFile,L"error:[%02d]nj_analyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");

    // Get predictive candidate list 2
    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,mergeReading2);
    Fprintf(logFile,L"\n");



    // Prediction 
    Fprintf(logFile,L" Prediction List\n");
    memcpy(&readingUtf16[0], mergeReading2, sizeof(readingUtf16));
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {

        memset(&result2[i], 0x00, sizeof(IWNN_RESULT));
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result2[i], NULL, ptrReading, &analyzeOption);       
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result2[i], sizeof(buf));
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]nj_get_candidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;    
        } else {
            Fprintf(logFile,L"error:[%02d]nj_analyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");


    //Merge candidate lists
    ret = mw::iwnn::IWNN::NjxMergeWordList(resultList, MAX_MERGE_LIST, NJ_MERGE_INIT, environment, (const IWNN_RESULT *)&result1, i);
    if (ret < 0){
        Fprintf(logFile,L"error:NjxMergeWordList-%04x-\n",ret);
        return -1;
    }
    ret = mw::iwnn::IWNN::NjxMergeWordList(resultList, MAX_MERGE_LIST, NJ_MERGE_NOT_EXIST, environment, (const IWNN_RESULT *)&result2, i);
    if (ret < 0){
        Fprintf(logFile,L"error:NjxMergeWordList-%04x-\n",ret);
        return -1;
    }
    for (i = 0; i < MAX_MERGE_LIST; i++){
        ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, resultList[i].result, sizeof(buf));
        if (ret < 0) {
            break;
        }
        Fprintf(logFile,L" [%2d]-",i);
        FprintUTF16BE(logFile,buf);
        Fprintf(logFile,L"\n");
    }

    Fprintf(logFile,L" Prediction List-End-\n");
    Fprintf(logFile,L"Get Merge List Demo -End-\n\n");
    Fprintf(logFile,L"-------------------------------------------------------\n");
    return i;
}

/*
 
               

 
 
 
                                   
                                   
 
 */
s16 demoChangeDicType(IWNN_CLASS *environment, IWNN_DIC_HANDLE handle, u8 direct)
{
    u16 ret;
    if ((environment == NULL)||(handle == NULL)) {
        return -1;
    }
    if ((direct != 0)&&(direct != 1)) {
        return -1;
    }


    Fprintf(logFile,L"-------------------------------------------------------\n");
    Fprintf(logFile,L"Change Dic Type Demo -Start-\n");
    ret = mw::iwnn::IWNN::NjxChangeDicType(handle, environment, direct);
    if (ret != 0) {
        Fprintf(logFile,L"error:NjxChangeDicType-%04x-\n", ret);
        return -1;
    } else {
        Fprintf(logFile,L"demoChangeDicType Demo -OK-\n");
    }
    Fprintf(logFile,L"Change Dic Type Demo -End-\n");
    Fprintf(logFile,L"-------------------------------------------------------\n");
    return ret;
}

/*
 

 

 
 
 

 
 */
u16 wcharStringLen(wchar_t *c) {

    u16 count = 0;

    while (*c++ != NULL) {
        count++;
    }
    return count;
}
