/*-----------------------------------------------------------------------*
 Project: Roman Input Demo: Conversion of Roman letter to KANA letter
 File:    RomanLetter.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <nn/types.h>
#include "RomanLetter.h"

typedef struct {
    wchar_t    *spell;            // Spelling
    wchar_t    *syllabary;        // Corresponding character
} ROMAN_TABLE;
const ROMAN_TABLE RomanLetterTbl[ ROMAN_LETTER_NUM ] = {
    // This input method must be implemented
    { L"a",    L"" },
    { L"i",    L"" },
    { L"u",    L"" },
    { L"e",    L"" },
    { L"o",    L"" },
    { L"ka",   L"" },
    { L"ki",   L"" },
    { L"ku",   L"" },
    { L"ke",   L"" },
    { L"ko",   L"" },
    { L"ga",   L"" },
    { L"gi",   L"" },
    { L"gu",   L"" },
    { L"ge",   L"" },
    { L"go",   L"" },
    { L"sa",   L"" },
    { L"si",   L"" },
    { L"shi",  L"" },
    { L"su",   L"" },
    { L"se",   L"" },
    { L"so",   L"" },
    { L"za",   L"" },
    { L"zi",   L"" },
    { L"ji",   L"" },
    { L"zu",   L"" },
    { L"ze",   L"" },
    { L"zo",   L"" },
    { L"ta",   L"" },
    { L"ti",   L"" },
    { L"chi",  L"" },
    { L"tu",   L"" },
    { L"tsu",  L"" },
    { L"te",   L"" },
    { L"to",   L"" },
    { L"da",   L"" },
    { L"di",   L"" },
    { L"du",   L"" },
    { L"de",   L"" },
    { L"do",   L"" },
    { L"na",   L"" },
    { L"ni",   L"" },
    { L"nu",   L"" },
    { L"ne",   L"" },
    { L"no",   L"" },
    { L"ha",   L"" },
    { L"hi",   L"" },
    { L"hu",   L"" },
    { L"fu",   L"" },
    { L"he",   L"" },
    { L"ho",   L"" },
    { L"ba",   L"" },
    { L"bi",   L"" },
    { L"bu",   L"" },
    { L"be",   L"" },
    { L"bo",   L"" },
    { L"pa",   L"" },
    { L"pi",   L"" },
    { L"pu",   L"" },
    { L"pe",   L"" },
    { L"po",   L"" },
    { L"ma",   L"" },
    { L"mi",   L"" },
    { L"mu",   L"" },
    { L"me",   L"" },
    { L"mo",   L"" },
    { L"ya",   L"" },
    { L"yu",   L"" },
    { L"yo",   L"" },
    { L"ra",   L"" },
    { L"ri",   L"" },
    { L"ru",   L"" },
    { L"re",   L"" },
    { L"ro",   L"" },
    { L"wa",   L"" },
    { L"wyi",  L"" },
    { L"wye",  L"" },
    { L"wo",   L"" },
    { L"kya",  L"" },
    { L"kyu",  L"" },
    { L"kyo",  L"" },
    { L"gya",  L"" },
    { L"gyu",  L"" },
    { L"gyo",  L"" },
    { L"sya",  L"" },
    { L"sha",  L"" },
    { L"syu",  L"" },
    { L"shu",  L"" },
    { L"syo",  L"" },
    { L"sho",  L"" },
    { L"zya",  L"" },
    { L"ja",   L"" },
    { L"zyu",  L"" },
    { L"ju",   L"" },
    { L"zyo",  L"" },
    { L"jo",   L"" },
    { L"tya",  L"" },
    { L"cha",  L"" },
    { L"tyu",  L"" },
    { L"chu",  L"" },
    { L"tyo",  L"" },
    { L"cho",  L"" },
    { L"dya",  L"" },
    { L"dyu",  L"" },
    { L"dyo",  L"" },
    { L"nya",  L"ɂ" },
    { L"nyu",  L"ɂ" },
    { L"nyo",  L"ɂ" },
    { L"hya",  L"Ђ" },
    { L"hyu",  L"Ђ" },
    { L"hyo",  L"Ђ" },
    { L"bya",  L"т" },
    { L"byu",  L"т" },
    { L"byo",  L"т" },
    { L"pya",  L"҂" },
    { L"pyu",  L"҂" },
    { L"pyo",  L"҂" },
    { L"mya",  L"݂" },
    { L"myu",  L"݂" },
    { L"myo",  L"݂" },
    { L"rya",  L"" },
    { L"ryu",  L"" },
    { L"ryo",  L"" },
    { L"sye",  L"" },
    { L"she",  L"" },
    { L"zye",  L"" },
    { L"je",   L"" },
    { L"tye",  L"" },
    { L"che",  L"" },
    { L"tsa",  L"" },
    { L"tse",  L"" },
    { L"tso",  L"" },
    { L"thi",  L"Ă" },
    { L"dhi",  L"ł" },
    { L"dhu",  L"ł" },
    { L"fa",   L"ӂ" },
    { L"fi",   L"ӂ" },
    { L"fe",   L"ӂ" },
    { L"fo",   L"ӂ" },
    { L"xa",   L"" },
    { L"xi",   L"" },
    { L"xu",   L"" },
    { L"xe",   L"" },
    { L"xo",   L"" },
    { L"xka",  L"" },
    { L"xke",  L"" },
    { L"xtu",  L"" },
    { L"xya",  L"" },
    { L"xyu",  L"" },
    { L"xyo",  L"" },
    { L"xwa",  L"" },
    { L"n\'",  L"" },
    { L"nn",   L"" },
    { L"n",    L"" },
    { L"-",    L"[" },
    // This input method is best implemented as an addition
    { L"ye",   L"" },
    { L"whi",  L"" },
    { L"wi",   L"" },
    { L"whe",  L"" },
    { L"we",   L"" },
    { L"who",  L"" },
    { L"va",   L"" },
    { L"vi",   L"" },
    { L"vu",   L"" },
    { L"ve",   L"" },
    { L"vo",   L"" },
    { L"vyu",  L"" },
    { L"kwa",  L"" },
    { L"qa",   L"" },
    { L"kwi",  L"" },
    { L"qi",   L"" },
    { L"kwe",  L"" },
    { L"qe",   L"" },
    { L"kwo",  L"" },
    { L"qo",   L"" },
    { L"gwa",  L"" },
    { L"jya",  L"" },
    { L"jyu",  L"" },
    { L"jyo",  L"" },
    { L"cya",  L"" },
    { L"cyu",  L"" },
    { L"cyo",  L"" },
    { L"tsi",  L"" },
    { L"thi",  L"Ă" },
    { L"dhi",  L"ł" },
    { L"thu",  L"Ă" },
    { L"twu",  L"Ƃ" },
    { L"dwu",  L"ǂ" },
    { L"hwa",  L"ӂ" },
    { L"hwi",  L"ӂ" },
    { L"hwe",  L"ӂ" },
    { L"hwo",  L"ӂ" },
    { L"fyu",  L"ӂ" },
    { L"hwyu", L"ӂ" },
    { L"xtsu", L"" },
    { L"^",    L"[" },
    // Other input methods
    { L"ca",   L"" },
    { L"ci",   L"" },
    { L"cu",   L"" },
    { L"ce",   L"" },
    { L"co",   L"" },
    { L"la",   L"" },
    { L"li",   L"" },
    { L"lu",   L"" },
    { L"le",   L"" },
    { L"lo",   L"" },
    { L"lka",  L"" },
    { L"lke",  L"" },
    { L"ltu",  L"" },
    { L"lya",  L"" },
    { L"lyu",  L"" },
    { L"lyo",  L"" },
    { L"lwa",  L"" },
    { L"ltsu", L"" },
    { L",",    L"A" },
    { L".",    L"B" },
};

s32  RomanLetterGetStringLength( wchar_t *str );
s32 RomanLetterCompareNString( wchar_t *str1, wchar_t *str2, s32 len );
void RomanLetterConcatenateString( wchar_t *str1, wchar_t *str2 );
void RomanLetterConcatenateNString( wchar_t *str1, wchar_t *str2, s32 len );
s32 RomanLetterCharConvert( wchar_t *str, wchar_t *dst );
s32 RomanLetterCharConvertEx( wchar_t *str, wchar_t *dst );

static int            RomanLetterBeforeCnt;
static wchar_t    RomanLetterBefore;

/*
 

 

 
 */
s32 RomanLetterGetStringLength( wchar_t *str )
{
    s32 cnt;

    cnt = 0;
    while ( str[ cnt ] )
    {
        cnt++;
    }

    return cnt;
}

/*
 

 
 
 

 
 */
s32 RomanLetterCompareNString( wchar_t *str1, wchar_t *str2, s32 len )
{
    s32 i;

    for ( i = 0 ; i < len ; i++ )
    {
        if ( ( ( ( str1[ i ] >= 'A' ) && ( str1[ i ] <= 'Z' ) ) ? ( str1[ i ] + 0x20 ) : str1[ i ] ) != str2[ i ] )
            return -1;
    }

    return 0;
}

/*
 

 
 
 
 
 */
void RomanLetterConcatenateString( wchar_t *str1, wchar_t *str2 )
{
    s32 i, len;

    i = RomanLetterGetStringLength( str1 );
    len = RomanLetterGetStringLength( str2 );

    while ( len )
    {
        str1[ i ] = *str2;
        i++;
        str2++;
        len--;
    }
}

/*
 

 
 
 

 
 */
void RomanLetterConcatenateNString( wchar_t *str1, wchar_t *str2, s32 len )
{
    s32 i;

    i = RomanLetterGetStringLength( str1 );

    while ( len )
    {
        str1[ i ] = *str2;
        i++;
        str2++;
        len--;
    }
}

/*
 

 
 

 

 
 */
s32 RomanLetterCharConvert( wchar_t *str, wchar_t *dst )
{
    s32 i, len;

    for ( i = 0 ; i < ROMAN_LETTER_NUM ; i++ )
    {
        len = RomanLetterGetStringLength( RomanLetterTbl[ i ].spell );
        if ( RomanLetterCompareNString( str, RomanLetterTbl[ i ].spell, len ) == 0 )
        {
            if ( i == ROMAN_LETTER_BASIS_NN )
            {
                if ( ( str[ len ] == 'a' ) || ( str[ len ] == 'A' )
                  || ( str[ len ] == 'i' ) || ( str[ len ] == 'I' )
                  || ( str[ len ] == 'u' ) || ( str[ len ] == 'U' )
                  || ( str[ len ] == 'e' ) || ( str[ len ] == 'E' )
                  || ( str[ len ] == 'o' ) || ( str[ len ] == 'O' )
                  || ( str[ len ] == '\'' ) )
                    continue;
                if ( ( str[ len ] == 'y' ) || ( str[ len ] == 'Y' ) )
                {
                    if ( str[ len + 1 ] == NULL )
                        return 0;
                    continue;
                }
                if ( str[ len ] == NULL )
                    return 0;
            }
            if ( i == ROMAN_LETTER_BASIS_N )
            {
                if ( ( str[ len ] == 'y' ) || ( str[ len ] == 'Y' ) )
                {
                    if ( str[ len + 1 ] == NULL )
                        return 0;
                    continue;
                }
                if ( str[ len ] == NULL )
                    continue;
            }
            if ( RomanLetterBeforeCnt > 0 )
            {
                s32 j;
                if ( RomanLetterBefore == str[ 0 ] )
                {
                    for ( j = 0 ; j < RomanLetterBeforeCnt ; j++ )
                        RomanLetterConcatenateNString( dst, L"", 1 );
                    RomanLetterBeforeCnt = 0;
                }
                else
                {
                    for ( j = 0 ; j < RomanLetterBeforeCnt ; j++ )
                        RomanLetterConcatenateNString( dst, &RomanLetterBefore, 1 );
                    RomanLetterBeforeCnt = 0;
                }
            }
            RomanLetterConcatenateString( dst, RomanLetterTbl[ i ].syllabary );
            return len;
        }
    }

    return 0;
}

/*
 

 
 

 

 
 */
void RomanLetterConvert( wchar_t *str, wchar_t *dst )
{
    s32 i, len;

    i = 0;
    RomanLetterBeforeCnt = 0;
    RomanLetterBefore = '\0';
    while ( str[ i ] != NULL )
    {
        len = RomanLetterCharConvert( &str[ i ], dst );
        if ( len == 0 )
        {
            if ( RomanLetterBeforeCnt > 0 )
            {
                if ( RomanLetterBefore != str[ i ] )
                {
                    RomanLetterConcatenateNString( dst, &str[ i - RomanLetterBeforeCnt ], RomanLetterBeforeCnt );
                    RomanLetterBeforeCnt = 0;
                }
            }
            RomanLetterBefore = str[ i ];
            RomanLetterBeforeCnt++;
            len = 1;
        }
        i += len;
    }
    if ( RomanLetterBeforeCnt > 0 )
    {
        RomanLetterConcatenateNString( dst, &str[ i - RomanLetterBeforeCnt ], RomanLetterBeforeCnt );
        RomanLetterBeforeCnt = 0;
    }
}

/*
 

 
 

 

 
 */
s32 RomanLetterCharConvertEx( wchar_t *str, wchar_t *dst )
{
    s32 i, len;

    for ( i = 0 ; i < ROMAN_LETTER_NUM ; i++ )
    {
        len = RomanLetterGetStringLength( RomanLetterTbl[ i ].spell );
        if ( RomanLetterCompareNString( str, RomanLetterTbl[ i ].spell, len ) == 0 )
        {
            if ( RomanLetterBeforeCnt > 0 )
            {
                s32 j;
                if ( RomanLetterBefore == str[ 0 ] )
                {
                    for ( j = 0 ; j < RomanLetterBeforeCnt ; j++ )
                        RomanLetterConcatenateNString( dst, L"", 1 );
                    RomanLetterBeforeCnt = 0;
                }
                else
                {
                    for ( j = 0 ; j < RomanLetterBeforeCnt ; j++ )
                        RomanLetterConcatenateNString( dst, &RomanLetterBefore, 1 );
                    RomanLetterBeforeCnt = 0;
                }
            }
            RomanLetterConcatenateString( dst, RomanLetterTbl[ i ].syllabary );
            return len;
        }
    }

    return 0;
}

/*
 

 
 

 

 
 */
void RomanLetterConvertEx( wchar_t *str, wchar_t *dst )
{
    s32 i, len;

    i = 0;
    RomanLetterBeforeCnt = 0;
    RomanLetterBefore = '\0';
    while ( str[ i ] != NULL )
    {
        len = RomanLetterCharConvertEx( &str[ i ], dst );
        if ( len == 0 )
        {
            if ( RomanLetterBeforeCnt > 0 )
            {
                if ( RomanLetterBefore != str[ i ] )
                {
                    RomanLetterConcatenateNString( dst, &str[ i - RomanLetterBeforeCnt ], RomanLetterBeforeCnt );
                    RomanLetterBeforeCnt = 0;
                }
            }
            RomanLetterBefore = str[ i ];
            RomanLetterBeforeCnt++;
            len = 1;
        }
        i += len;
    }
    if ( RomanLetterBeforeCnt > 0 )
    {
        RomanLetterConcatenateNString( dst, &str[ i - RomanLetterBeforeCnt ], RomanLetterBeforeCnt );
        RomanLetterBeforeCnt = 0;
    }
}

